from wk.extra.node import *
from wk.extra.node import boostrap as bs

class SearchPageBase(Html):
    environment = dict(
        blue='#1D93EC'
    )
    def __init__(self):
        super().__init__()
        self.__call__([
            Head()([
                Meta(encoding='utf-8'),
                Meta(name="viewport",content="width=device-width, initial-scale=1, shrink-to-fit=no"),
                Title()(
                    Var(name="title",type='text')("高中学习资源下载")
                ),
                QLinks.bootstrap()
            ]),
            Body()([
                Var(name='body')([
                    Div(style="max-width:30em;",_class="m-auto p-3 pt-5  max-vh-30")(
                        Form(action='/search',_class="m-auto")([
                            '''
                            <div class="input-group mb-3">
  <input type="text" class="form-control" placeholder="{{placeholder  or '输入关键词，如“数学立体几何”'}}" name="search-keywords", value="{{input_fill or ''}}">
  <div class="input-group-append">
    <button class="btn btn-success" type="submit" id="basic-addon2">Search</button>
  </div>
</div>
                            '''
                        ])
                    ),
                    Div(_class="list-group p-2 m-auto text-center",style="max-width:40em;")(
                        Var(name="search_result")(
                            Div()("Nothing to show.")
                        )
                    )
                ]),
                QScripts.jquery(),
                QScripts.popper(),
                QScripts.bootstrap(),
            ])
        ])


class FileCard(Div):
    def __init__(self,file):
        super().__init__(_class='card')
        self.__call__([
            # A(href=file['url'])([
            #     bs.Badges.secondary(file['name'])
            # ]),
            # bs.Badges.success(file['size']),
            '''
      <li class="list-group-item" ><a href="{url}">{name}</a><span class="badge badge-secondary ml-2">{size}</span></li>
 '''.format(name=file['name'],size=file['size'],url=file['url'])
        ])


from wk.extra.linux import *
from wk.io import load_simple_config
if __name__ == '__main__':
    ip = get_local_ip()
    # ip = '127.0.0.1'
    port = 80
    clean_port(port)
    config=load_simple_config(
        # './windows.cfg',
        './config.cfg',
    )
    from wk.web.applications.knowledge_platform import KnowledgePlatform
    app = KnowledgePlatform(__name__, config=config)
    app.run(host=ip,port=port)



from wk.extra.node import *


class Urls:
    class js:
        bootstrap = 'https://cdn.staticfile.org/twitter-bootstrap/4.5.0/js/bootstrap.min.js'
        jquery = 'https://cdn.staticfile.org/jquery/3.5.1/jquery.min.js'
        vue = 'https://cdn.staticfile.org/vue/2.6.11/vue.min.js'
        react = 'https://cdn.staticfile.org/react/16.13.1/cjs/react.production.min.js'
        popper = 'https://cdn.staticfile.org/popper.js/2.4.0/umd/popper.min.js'
        marked = 'https://cdn.staticfile.org/marked/1.1.0/marked.min.js'
        jwerty = 'https://cdn.staticfile.org/jwerty/0.3.2/jwerty.min.js'
        font_awesome = 'https://cdn.staticfile.org/font-awesome/5.13.0/js/all.min.js'
        jgrowl = 'https://cdn.staticfile.org/jquery-jgrowl/1.4.7/jquery.jgrowl.min.js'
        alertify = 'https://cdn.staticfile.org/AlertifyJS/1.13.1/alertify.min.js'
        zico = 'http://ico.z01.com/zico.min.js'
        storejs = 'https://cdn.staticfile.org/store.js/2.0.12/store.everything.min.js'

        class highlightjs:
            highlightjs = 'https://cdn.staticfile.org/highlight.js/10.0.3/highlight.min.js'

            class languages:
                java = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/java.min.js'
                javascript = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/javascript.min.js'
                julia = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/julia.min.js'
                kotlin = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/kotlin.min.js'
                makefile = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/makefile.min.js'
                markdown = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/markdown.min.js'
                matlab = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/matlab.min.js'
                php = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/php.min.js'
                python = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/python.min.js'
                c = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/c.min.js'
                c_like = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/c-like.min.js'
                cpp = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/cpp.min.js'
                css = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/css.min.js'
                http = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/http.min.js'
                go = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/go.min.js'
                rust = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/rust.min.js'
                shell = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/shell.min.js'
                ruby = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/ruby.min.js'
                sql = 'https://cdn.staticfile.org/highlight.js/10.0.3/languages/sql.min.js'

    class css:
        bootstrap = 'https://cdn.staticfile.org/twitter-bootstrap/4.5.0/css/bootstrap.min.css'
        font_awesome = 'https://cdn.staticfile.org/font-awesome/5.13.0/css/all.min.css'
        jgrowl = 'https://cdn.staticfile.org/jquery-jgrowl/1.4.7/jquery.jgrowl.min.css'
        alertify = 'https://cdn.staticfile.org/AlertifyJS/1.13.1/css/alertify.min.css'
        zico = 'https://ico.z01.com/zico.min.css'

        class highlightjs:
            default = 'https://cdn.staticfile.org/highlight.js/10.0.3/styles/default.min.css'
            github = 'https://cdn.staticfile.org/highlight.js/10.0.3/styles/github.min.css'
            rainbow = 'https://cdn.staticfile.org/highlight.js/10.0.3/styles/rainbow.min.css'
            ocean = 'https://cdn.staticfile.org/highlight.js/10.0.3/styles/ocean.min.css'
            tomorrow = 'https://cdn.staticfile.org/highlight.js/10.0.3/styles/tomorrow.min.css'

            class atom:
                dark = 'https://cdn.staticfile.org/highlight.js/10.0.3/styles/atom-one-dark.min.css'
                light = 'https://cdn.staticfile.org/highlight.js/10.0.3/styles/atom-one-light.min.css'


class QUrlScript(Script): _attrs = dict(type='text/javascript')


class QStyleLink(Link): _attrs = dict(rel="stylesheet", type="text/css")


class QLinks:
    class bootstrap(QStyleLink): _attrs = dict(href=Urls.css.bootstrap)

    class font_awesome(QStyleLink): _attrs = dict(href=Urls.css.font_awesome)

    class jgrowl(QStyleLink): _attrs = dict(href=Urls.css.jgrowl)

    class alertify(QStyleLink): _attrs = dict(href=Urls.css.alertify)

    class zico(QStyleLink): _attrs = dict(href=Urls.css.zico)


class QScripts:
    class jquery(QUrlScript): _attrs = dict(src=Urls.js.jquery)

    class popper(QUrlScript): _attrs = dict(src=Urls.js.popper)

    class bootstrap(QUrlScript): _attrs = dict(src=Urls.js.bootstrap)

    class jwerty(QUrlScript): _attrs = dict(src=Urls.js.jwerty)

    class font_awesome(QUrlScript): _attrs = dict(src=Urls.js.font_awesome)

    class jgrowl(QUrlScript): _attrs = dict(src=Urls.js.jgrowl)

    class alertify(QUrlScript): _attrs = dict(src=Urls.js.alertify)

    class zico(QUrlScript): _attrs = dict(src=Urls.js.zico)

    class storejs(QUrlScript): _attrs = dict(src=Urls.js.storejs)


class QWindow(Div):
    _attrs = dict(_class='QWindow')

    def __init__(self, width="300px", height="300px", position="static"):
        super().__init__(
            style="width:{width};height:{height};position:{position};background-color:{{blue}};"
                  "border:green 2px solid;color:white;display:flex;".format(width=width, height=height,
                                                                            position=position)
        )
        self.__call__([
            Var(name='content')("This is  a window.")
        ])


class QCenterDiv(Div):
    _attrs = dict(style="text-align:center;")


class QListItem(Div):
    def __init__(self):
        super().__init__(
            style="width:100%;padding:20px;box-sizing:border-box;"
        )


class QList(Div):
    _attrs = dict(_class="QList")

    def __init__(self, color="black"):
        super().__init__(
            style='width:100%;height:100%;overflow:scroll;color:{color};box-sizing:border-box;'.format(color=color)
        )
        self.__call__(
            Var(name="items")([
                QListItem()([
                    QCenterDiv()("This is a demo QListItem.")
                ])
            ])
        )


class QDocument(Html):
    environment = dict(
        blue='#1D93EC'
    )

    def __init__(self):
        super().__init__()
        self.__call__([
            Head()([
                Meta(encoding='utf-8'),
                Title()(
                    Var(name="title", type='text')("Hello World !")
                ),
                QLinks.bootstrap(),
                QScripts.jquery(),
                QScripts.bootstrap(),
            ]),
            Var(name="body")([
                Body()([
                    QWindow(height="150px").compile(
                        content=Form(action='/search')([
                            Input(type="text", placeholder="{{placeholder  or 'Input something.'}}",
                                  name="search-keywords", value="{{input_fill or ''}}"),
                            Button(type='submit')('Search')
                        ])
                    ),
                    QWindow(width="300px", height="400px").compile(
                        content=QList()([
                            Var(name="search_result")([
                                QListItem()("Hi")
                            ])
                        ])
                    )
                ])
            ])
        ])


def html(args):
    default_args = dict(
        title='Html Demo',
        contents=''
    )
    smart_update_dict(default_args, args)
    args = default_args
    return Html()([
        Head()([
            Meta(charset='utf-8'),
            Title()(args['title']),
        ]),
        Body()(args['contents'])
    ])


def form_demo(args={}):
    default_args = dict(
        action='', method='post', style='background-color:#1D93EC;color:white',
        username=dict(type='text', name='username', placeholder='username'),
        password=dict(type='password', name='password', placeholder='password'),
        submit_btn=dict(type='submit', style="background-color:#1D93EC;color:white")
    )
    smart_update_dict(default_args, args)
    args = default_args
    username = args.pop('username')
    password = args.pop('password')
    submit_btn = args.pop('submit_btn')

    return Form(**args)([
        Label()(Input(**username)),
        Label()(Input(**password)),
        Button(**submit_btn)("Submit")
    ])


def demo():
    f = r'D:\work\wk\data/test.html'
    x = Node()('hi')
    # x = html("hi")
    # x = form_demo()
    blue = "#1D93EC"
    x = html(dict(
        contents=[
            Div(style='background-color:{{blue}};color:white;min-height:50px')([
                form_demo(dict(
                    action="http:/127.0.0.1:80/",
                    username=dict(placeholder="input your username...")
                ))
            ])
        ]
    ))
    with open(f, 'w') as fp:
        fp.write(x.render(blue=blue))
    print(x)


def demo2():
    f = r'D:\work\wk\data/test.html'
    doc = QDocument()
    doc.compile()
    x = doc.render()
    doc.to_file(f)
    # with open(f,'w') as fp:
    #     fp.write(x)
    print(x)


if __name__ == '__main__':
    demo2()



from wk import web, join_path, FakeOS, Folder, PointDict
from . import pages, engines
import os, shutil, glob, json
import wk

'''
Site Structure:
    /               :search home
    /tree           :knowledge tree
    /f      :search files, to view files
    /u/<id>         :user home page
    /p/<id>         :read article
    /join           :sign up
    /login          :login
    /logout         :logout
    
    /static
    /files
    /pkg-resource
Contents:
    文章
    题目
    文件
    问答
    知识点
    知识树
**已完成Next Step: 
    实现文章的发布、删除
    publish: 
    delete:
    private:
    三张表：用户文章列表、公开文章列表、回收站文章列表
    文档引擎三？四？张表：公开文章列表、回收站文章列表、所有文章列表
    每个用户文章列表: 私有、公开、回收站
    用户发布文章时：
        
    用户删除文章时：
Bug:
    用户退出登录后，访问其他用户主页时，导航栏还有其他用户的头像,没有对其他用户进行区分
Next:
    已发布文章取消发布?
    查看已删除的文章？恢复文章？彻底删除？
    搜索引擎清空索引的方法？
    文档引擎更新方法？清空方法？
    
NextNextStep:
    实现文章的推荐
    实现文章的组织方式（结构化的系统）
    文件的筛选方式
Future:
    问答、视频、评论系统、内容自动生成
    
'''


class KnowledgePlatform(web.MyBlueprint):
    url_prefix = '/'

    def __init__(self, import_name, config={}, *args, **kwargs):
        super().__init__(import_name, *args, **kwargs)
        self.config = config
        serve_root = config['serve_root']

        class ENGINE:
            IndexEngine = engines.IndexEngine
            SearchEngine = engines.SearchEngine
            StaticStorageEngine = engines.StaticStorageEngine
            DocumentStorageEngine = engines.DocumentStorageEngine
            RecommenderSystem=engines.RecommenderSystem

        class PATH:
            serve_root = config['serve_root']
            user_manager = serve_root + '/user_manager'
            static_storage_engine = serve_root + '/static_storage_engine'
            extra_static_dirs = [config['extra_static_dir']]
            index_engine = serve_root + '/index_engine'
            document_storage_engine = serve_root + '/document_storage_engine'

        class Sitemap:
            home = '/'
            signup = '/join'
            login = '/login'
            logout = '/logout'
            search_file = '/files'
            upload = '/upload'
            editor = '/editor'

            class file:
                like = '/file/op/like'
                dislike = '/file/op/dislike'

            class article:
                # new = '/article/new'
                edit = '/article/edit'
                new = edit
                publish = '/article/publish'
                delete = '/article/delete'

            class prefix:
                article = '/p'
                user = '/u'
                static_files = '/f'
                pkg_resource = '/pkg-resource'

        class URL:
            pkg_resource= join_path(self.url_prefix,'pkg-resource')
            site_resource =join_path(pkg_resource,'sites','zspt')
            logo=join_path(site_resource,'img','logo1.png')
            favicon=join_path(site_resource,'img','favicon.ico')
            home = join_path(self.url_prefix, Sitemap.home)
            signup = join_path(self.url_prefix, Sitemap.signup)
            login = join_path(self.url_prefix, Sitemap.login)
            logout = join_path(self.url_prefix, Sitemap.logout)
            search_file = join_path(self.url_prefix, Sitemap.search_file)
            upload = join_path(self.url_prefix, Sitemap.upload)
            editor = join_path(self.url_prefix, Sitemap.editor)
            class getter:
                pkg_resource=lambda x:join_path(self.url_prefix,'pkg-resource',x)
                icon=lambda x:join_path(self.url_prefix,'pkg-resource/imgs/icon',x)
                class site:
                    css = lambda x: join_path(self.url_prefix, 'pkg-resource', 'sites/zspt/css')
                    img = lambda x: join_path(self.url_prefix, 'pkg-resource', 'sites/zspt/img')
                    js = lambda x: join_path(self.url_prefix, 'pkg-resource', 'sites/zspt/js')

                class article:
                    view = lambda id: join_path(self.url_prefix, Sitemap.prefix.article, id)
                class seach_file:
                    query=lambda x:join_path(Sitemap.search_file,'?q='+x)

            class file:
                like=join_path(self.url_prefix,Sitemap.file.like)
                dislike=join_path(self.url_prefix,Sitemap.file.dislike)

            class article:
                new = join_path(self.url_prefix, Sitemap.article.new)
                edit = join_path(self.url_prefix, Sitemap.article.edit)
                publish = join_path(self.url_prefix, Sitemap.article.publish)
                delete = join_path(self.url_prefix, Sitemap.article.delete)

                class getter:
                    edit = lambda id: join_path(self.url_prefix, Sitemap.article.edit) + '?article_id=%s' % (id)
                    publish = lambda id: join_path(self.url_prefix, Sitemap.article.publish) + '?article_id=%s' % (id)
                    delete = lambda id: join_path(self.url_prefix, Sitemap.article.delete) + '?article_id=%s' % (id)
                    view =lambda id:join_path(self.url_prefix,Sitemap.prefix.article,id)

            class prefix:
                article = join_path(self.url_prefix, Sitemap.prefix.article)
                user = join_path(self.url_prefix, Sitemap.prefix.user)
                static_files = join_path(self.url_prefix, Sitemap.prefix.static_files)
                pkg_resource = join_path(self.url_prefix, Sitemap.prefix.pkg_resource)

        self.serve_root = serve_root
        self.usman = web.UserManager(PATH.user_manager,
                                     User=engines.User,
                                     home_url=URL.home,
                                     signup_url=URL.signup,
                                     login_url=URL.login,
                                     logout_url=URL.logout, )
        self.usman.register(self)
        self.index_engine = ENGINE.IndexEngine(PATH.index_engine)
        self.search_engine = ENGINE.SearchEngine(self.index_engine)
        self.static_storage_engine = ENGINE.StaticStorageEngine(PATH.static_storage_engine, check_when_start=True,
                                                                include_dirs=PATH.extra_static_dirs)
        self.document_engine = ENGINE.DocumentStorageEngine(PATH.document_storage_engine,
                                                            index_engine=self.index_engine, usman=self.usman,
                                                            id2url=lambda id: join_path(URL.prefix.article, id))
        self.recommender_system=ENGINE.RecommenderSystem(self.document_engine)
        app = self

        class Context(web.Context):
            def __init__(self, *args, **kwargs):
                super().__init__(*args)
                self.update(
                    builtin=PointDict(
                        len=len,
                    ),
                    utils=PointDict(
                        wk=wk,
                        jsonify=web.jsonify,
                        to_datetime_str=wk.to_datetime_str,
                        to_date_str=wk.to_date_str,
                    ),
                    user=None,
                    URL=URL,
                    get_user=app.usman.get_user,
                    get_article=app.document_engine.get,
                    get_user_link=lambda id: join_path(URL.prefix.user, id),
                    get_article_link=lambda aid: join_path(URL.prefix.article, aid),
                )
                self.update(**kwargs)

        # ###################Site Home Page####################
        @self.route(Sitemap.home, methods=['get'])
        @self.usman.get_user_context(arg_name='user')
        @web.parse_args
        def do_search(search_keywords, user):
            context = Context(user=user)
            if not search_keywords:
                doc_ids = self.recommender_system.recommend_latest(n=20)
                docs = [self.document_engine.get(doc_id) for doc_id in doc_ids]
                docs = [pages.ArticleCard().render(article=doc,context=context) for doc in docs]
                return pages.SearchResultPage().compile(result=docs).render(context=context)
            else:
                doc_ids = self.search_engine.search(search_keywords)
                docs = [self.document_engine.get(doc_id) for doc_id in doc_ids]
                for i,doc in enumerate(docs):
                    if doc is None:
                        docs.remove(doc)
                docs = [pages.ArticleCard().render(article=doc,context=context) for doc in docs]
                return pages.SearchResultPage().compile(result=docs).render(context=context)

        # ###################View Article####################
        @self.route(Sitemap.prefix.article + '/<string:id>')
        @self.usman.get_user_context(arg_name='user')
        def do_article(id, user):
            context = Context(user=user)
            doc = self.document_engine.get(id)
            return pages.ArticlePage().render(context=context,article=doc)

        # ###################Visit User Home Page####################
        @self.route(Sitemap.prefix.user + '/<string:id>')
        @self.usman.get_user_context(arg_name='user')
        def do_user(id, user):
            user_visit = self.usman.get_user(id)
            if not user_visit:
                return pages.ErrorPage(message='User Not Found.').render()
            user_visit['articles'] = [self.document_engine.get_info(doc_id) for doc_id in user_visit['articles']]

            if not user or id != user['id']:
                context = Context(user_visit=user_visit)
                return pages.UserVisitPage().render(context=context)
            else:
                context = Context(user=user_visit)
                # print(user_visit)
                return pages.UserHomePage().render(context=context)

        # ###################Article New Edit Publish Delete######################
        @self.route(Sitemap.article.new, methods=['post', 'get'])
        @self.usman.login_required
        @self.usman.get_user_context(arg_name='user')
        @web.parse_from(web.get_form, web.get_json, web.get_url_args)
        def do_new_edit_article(article_id, object, user):
            document = object
            context = Context(user=user, article=None)
            if article_id:
                # 编辑文章
                if not user.is_own_article(article_id):
                    return pages.NoPermissionPage().render(context=context)
                if document:
                    # post
                    self.document_engine.update_document(article_id, document)
                    return web.ActionRedirect(location=URL.article.getter.view(article_id), message="上传成功!").jsonify()
                else:
                    # get
                    document = self.document_engine.get(article_id)
                    context.article = document
                    return pages.EditArticlePage().render(context=context)
            else:
                # 创建新文章
                doc = object
                if doc is None:
                    # get
                    return pages.EditArticlePage().render(context=context)
                else:
                    # post
                    print(doc)
                    res = self.document_engine.check(doc)
                    if not res['success']:
                        return web.jsonify(res)
                    ret = self.document_engine.new(user, doc)
                    article_id=ret['id']
                    print("saved, info : %s" % (ret))
                    return web.ActionRedirect(location=URL.article.getter.view(article_id), message="上传成功!").jsonify()

        ###################Publish Article#####################
        @self.route(Sitemap.article.publish, methods=['post', 'get'])
        @self.usman.login_required
        @self.usman.get_user_context(arg_name='user')
        @web.parse_from(web.get_form, web.get_json, web.get_url_args)
        def do_publish_article(article_id, user):
            context = Context(user=user, article=None)
            assert article_id, user
            if not user.is_own_article(article_id):
                return pages.NoPermissionPage().render(context=context)
            if not self.document_engine.is_private(article_id):
                return web.StatusErrorResponse(message='你不能发布该文章,该文章可能已经发布').jsonify()
            url = self.document_engine.publish(user, article_id)
            # return web.ActionRedirect(location=url, message='发布成功!').jsonify()
            return pages.PublishSuccessPage(message='发布成功！', link=url, link_text='点击前往查看').render(context=context)

        ###################Delete Article#####################
        @self.route(Sitemap.article.delete, methods=['post', 'get'])
        @self.usman.login_required
        @self.usman.get_user_context(arg_name='user')
        @web.parse_from(web.get_form, web.get_json, web.get_url_args)
        def do_delete_article(article_id, user):
            context = Context(user=user, article=None)
            assert article_id, user
            if not user.is_own_article(article_id):
                return pages.NoPermissionPage().render(context=context)
            ret = self.document_engine.delete_article(user, article_id)
            return web.ActionRefresh(message=ret or '删除成功').jsonify()

        # ##################Search Files##########################
        @self.route(Sitemap.search_file + '/', methods=['post', 'get'])
        @self.usman.get_user_context(arg_name='user')
        @web.parse_args
        def do_search_files(user):
            args = dict(web.get_url_args())
            q = None
            if 'q' in args.keys():
                q = args.pop('q')

            subjects = dict(
                chinese='语文', math='数学', english='英语', physics='物理', chemistry='化学', biology='生物', history='历史',
                politics='政治', geography='地理'
            )
            if 'subject' in args:
                args['subject'] = subjects[args['subject']]
            categories={
                '语文':{
                    '知识点':{
                        '必修一':None,
                        '必修二':None,
                        '必修三':None,
                    },
                    '学习方法':None,
                    '思维导图':None,
                },
                '数学':{
                    '知识点': {
                        '必修一': None,
                        '必修二': None,
                        '必修三': None,
                    },
                    '学习方法': None,
                    '思维导图': None,
                },
                '英语':None,
            }
            context = Context(
                user=user,
                categories=json.dumps(categories,ensure_ascii=False,indent=2),
                recommendations=[],
                filters=[
                    PointDict(
                        name='subject',
                        title='科目',
                        choices=[PointDict(title=v, name=k) for k, v in subjects.items()]
                    )
                ]
            )

            if not q and not args:
                return pages.FileSearchPageBase().render(context=context)
            filters = args
            result = self.static_storage_engine.select(**filters)
            if q:
                result = self.static_storage_engine.search(q, candidates=result)
            result = [self.static_storage_engine.get_file_info(file) for file in result]
            for file in result:
                file['url'] = join_path(URL.prefix.static_files, file['id'])
            print("所搜到%s条结果"%(len(result)))
            x = pages.FileSearchPageBase().compile().render(input_fill=q,
                                                            files=result, context=context)
            return x

        # ###################File Get And Upload######################

        @self.route(Sitemap.prefix.static_files + '/<string:id>', methods=['get'])
        def do_get_static_files(id):
            return self.static_storage_engine.send_file(id)

        @self.route(Sitemap.upload, methods=['get', 'post'])
        @self.usman.get_user_context(arg_name='user')
        @web.parse_files
        def do_upload(file, image, user):
            context = Context(user=user)
            files=dict(web.request.files)
            # print(file,image,files)
            if file:
                info = self.static_storage_engine.saveFile(file)
                return web.jsonify({'link': URL.prefix.static_files + '/' + info['id']})
            elif image:
                info = self.static_storage_engine.saveImage(image)
                return web.jsonify({'link': URL.prefix.static_files + '/' + info['id']})
            else:
                return pages.UploadPage().render(context=context)
        # ###################File Operations######################
        @self.route(URL.file.like)
        @self.usman.login_required
        @web.parse_args
        def do_file_like(file_id,user):
            if not file_id:
                return ''
            file=self.static_storage_engine.get(file_id)
            file['zan']+=1
            self.static_storage_engine.update_file(file['id'],file)
            return web.StatusSuccessResponce(message='点赞成功')
        @self.route(URL.file.dislike)
        @self.usman.login_required
        @web.parse_args
        def do_file_dislike(file_id,user):
            if not file_id:
                return ''
            file=self.static_storage_engine.get(file_id)
            file['cai']+=1
            self.static_storage_engine.update_file(file['id'],file)
            return web.StatusSuccessResponce(message='点赞成功')




class LoggerFile:
    def __init__(self,path,recreate=False):
        self.path=path
        if recreate:
            self.recreate()
    def recreate(self):
        with open(self.path,'w') as f:
            pass
    def _push(self,text):
        with open(self.path,'a') as f:
            f.write(text)
    def log(self,*msgs):
        print(*msgs)
        self.logtofile(*msgs)
    def logtofile(self,*msgs):
        msgs = [str(msg) if hasattr(msg, '__str__') else repr(msg) for msg in msgs]
        msgs = ' '.join(msgs)+'\n'
        self._push(msgs)




from dulwich import porcelain as git
from dulwich.repo import Repo
import os, shutil, glob

default_remote_location= 'https://OpenGitspace:Gitspace@123456@gitee.com/OpenGitspace/meta'
defalut_user_location='https://OpenGitspace:Gitspace@123456@gitee.com/OpenGitspace/'

def get_default_remote_location(repo):
    return defalut_user_location+repo

class errors:
    class GitSpaceError(Exception):
        def __init__(self, *args):
            super().__init__(*args)


def is_git_dir(path):
    if os.path.exists(path) and os.path.isdir(path):
        dot_git_dir = path + '/.git'
        if os.path.exists(dot_git_dir) and os.path.isdir(dot_git_dir):
            return True
    return False


def is_empty_dir(path):
    assert os.path.exists(path) and os.path.isdir(path)
    if len(os.listdir(path)):
        return False
    else:
        return True

def to_string_iterable(iterable):
    lis=[s.decode() if isinstance(s,bytes) else s for s in iterable]
    return iterable.__class__(lis)
class GitRepo(Repo):
    def __init__(self,path):
        super().__init__(path)

    def clean(self):
        for name in os.listdir(self.path):
            if name=='.git':continue
            path=self.path+'/'+name
            if os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)
    def active_branch(self):
        return git.active_branch(self).decode()
    def checkout_branch(self,branch='master'):
        git.update_head(self,branch)
        self.clean()
        co_ref = b'HEAD'
        repo_path=self.path
        from dulwich.repo import Repo
        from dulwich.index import build_index_from_tree
        repo = Repo(repo_path)
        indexfile = repo.index_path()
        obj_sto = repo.object_store
        tree_id = repo[co_ref].tree
        build_index_from_tree(repo_path, indexfile, obj_sto, tree_id)
        x=list(obj_sto.iter_tree_contents(tree_id))
        x=[obj_sto.iter_tree_contents(tree_id)]
    def branch_create(self,name):
        git.branch_create(self,name)
    def pull(self, remote_location, branch):
        import logging
        logging.warning("Pulling from %s:%s" % (remote_location, branch))
        git.pull(self, remote_location, branch)
    def branch_list(self):
        return list(to_string_iterable(git.branch_list(self)))
    def status(self,silent=False):
        repo = self
        msg = git.status(repo)
        if not silent:
            print(msg)
        return msg
    def add_all(self):
        repo = self
        status=git.status(self)
        paths = status.untracked + status.unstaged
        paths = [p.decode() if isinstance(p, bytes) else p for p in paths]
        paths = [self.path + '/' + p for p in paths]
        if not paths:
            import logging
            logging.warning('Working tree is clean. Nothing to add.')
            return
        paths.append(self.path)
        git.add(repo, paths)
    def commit(self,msg='commit somthing'):
        git.commit(self, msg)
    def push(self, remote_location, branch):
        import logging
        logging.warning("Pushing to %s:%s"%(remote_location,branch))
        git.push(self, remote_location, branch)


def clone(src, path=None, overwrite=False,branch='master'):
    path = path or os.path.basename(src).rsplit('.git', maxsplit=1)[0]
    if os.path.exists(path) and overwrite:
        shutil.rmtree(path)
    assert not os.path.exists(path) or is_empty_dir(path)
    git.clone(src, path)
    repo=GitRepo(path)
    if not branch=='master':
        repo.pull(remote_location=src,branch=branch)
    return repo


import os,shutil,glob,json,pickle,re,time
from wk.basic import get_relative_path,standard_path,split_path,PowerDirPath,DirPath
from ..fsutils import copy_dir,copy_file
from .ofiles import ObjectFile,SimpleListFile,SimpleConfigFile,NumberFile,DictFile
class DirDict(object):
    def __init__(self,name=None,realpath=None,*args,**kwargs):
        if realpath and os.path.exists(realpath):
            realpath=os.path.abspath(realpath)
        self.realpath = realpath
        if realpath:
            name=name or os.path.basename(realpath)
        self.name=name
        self.hooked=realpath is not None
    def loadjson(self,fn,encoding='utf-8'):
        path = self.join(fn)
        with open(path,'r',encoding=encoding) as f:
            return json.load(f)
    def dumpjson(self,fn,obj,encoding='utf-8'):
        path=self.join(fn)
        with open(path,'w',encoding=encoding) as f:
            json.dump(obj,f)
    def savetxt(self,fn,string,encoding='utf-8'):
        path=self.join(fn)
        with open(path,'w',encoding=encoding) as f:
            f.write(string)
    def appendTo(self,path):
        if not isinstance(path,DirDict):
            path=DirDict(realpath=path)
        path.append(self)
    def append(self,path):
        if not isinstance(path,DirDict):
            path=DirDict(path)
        if not path.exists():
            self.makedirs(path.name)
        else:
            self.copy(path)
    def copy(self,path):
        assert self.exists()
        if not isinstance(path,DirDict):
            path=DirDict(realpath=path)
        assert path.exists()
        new_path=self.join(path.name)
        assert not os.path.exists(new_path)

        if os.path.isdir(path.realpath):
            copy_dir(path.realpath,new_path)
        else:
            shutil.copy(path.realpath,new_path)
    def makedirs(self,path):
        assert self.hooked
        assert self.exists()
        path = self.join(path)
        os.makedirs(path)
    def mkdir(self,name):
        assert self.hooked
        assert self.exists()
        assert self.secure(name)
        path=self.join(name)
        os.mkdir(path)

    def join(self,path):
        return self.realpath+'/'+path
    def listdir(self):
        return os.listdir(self.realpath)
    def exists(self):
        if not self.realpath:return False
        return os.path.exists(self.realpath)
    def isdir(self):
        return os.path.isdir(self.realpath)
    def isfile(self):
        return os.path.isfile(self.realpath)
    def ismount(self):
        return os.path.ismount(self.realpath)
    def islink(self):
        return os.path.islink(self.realpath)
    def getsize(self):
        return os.path.getsize(self.realpath)
    def getatime(self):
        return os.path.getatime(self.realpath)
    def getmtime(self):
        return os.path.getmtime(self.realpath)
    def getctime(self):
        return os.path.getctime(self.realpath)
    def secure(self,name):
        escapes=['/','\\']
        for ch in escapes:
            if ch in name:
                return False
        return True
    @classmethod
    def from_realpath(cls,path):
        realpath=os.path.abspath(path)
        name=os.path.basename(path)
        return cls(name=name,realpath=realpath)

class FakeOS:
    def __init__(self,path):
        if  path and os.path.exists(path):
            path=os.path.abspath(path)
        self.path=standard_path(path) if path else path
        self.name=os.path.basename(self.path)
        self.cache={}
    def _relpath(self,root,path):
        return get_relative_path(root,path)
    def _standard_path(self,*args,**kwargs):
        return standard_path(*args,**kwargs)
    def _split_path(self,*args,**kwargs):
        return split_path(*args,**kwargs)
    def _fakepath(self,path):
        return get_relative_path(self.path,path)
    def _truepath(self,path):
        if not self.path:
            return standard_path(path)
        return self.path+'/'+standard_path(path)
    def search(self,keywords,match_all=True):
        if not 'files' in self.cache.keys():
            self.cache['files']=self.glob('./**/*.*',recursive=True)
            self.cache['last_refresh_time']=time.time()
        elif time.time()-self.cache['last_refresh_time']>5*60*60:
            self.cache['files'] = self.glob('./**/*.*', recursive=True)
            self.cache['last_refresh_time'] = time.time()
        fs=self.cache['files']
        if match_all:
            for word in keywords:
                fs=list(filter(lambda f:re.findall(word,f),fs))
            return fs
        else:
            def match(text,ptns):
                for ptn in ptns:
                    if re.findall(ptn,text):return True
                return False
            fs=list(filter(lambda f:match(f,keywords),fs))
            return fs

    def glob(self,pathname='./',recursive=False):
        pathname=self._truepath(pathname)
        fs=glob.glob(pathname=pathname,recursive=recursive)
        fs=[self._fakepath(path) for path in fs]
        return fs
    def tranverse_info(self,path,depth=-1,format=True):
        return self.info(path,depth,format)
    def info(self, path,depth=2,format=True):
        path = self._truepath(path)
        return PowerDirPath(path).tranverse_info(depth=depth,format=format)
    def open(self, file, mode='r',encoding='utf-8'):
        file = self._truepath(file)
        return open(file, mode,encoding=encoding)
    def openDB(self,path):
        path=self._truepath(path)
        from wk import BackupDB
        return BackupDB(path)
    def openFiledict(self,path):
        path=self._truepath(path)
        from wk import FileDict
        return FileDict(path)
    def openObjectfile(self,path):
        path=self._truepath(path)
        return ObjectFile(path)
    def openSimpleListfile(self,path):
        path=self._truepath(path)
        return SimpleListFile(path)
    def openFolder(self,path):
        path=self._truepath(path)
        from wk import Folder
        return Folder(path)
    def dir(self,name):
        path=self._truepath(name)
        p=PowerDirPath(path)
        p.todir()
        return p
    def file(self,name):
        path = self._truepath(name)
        p = PowerDirPath(path)
        p.tofile()
        return p
    def read(self, fp, mode='r', encoding='utf-8'):
        fp=self._truepath(fp)
        with open(fp, mode=mode, encoding=encoding) as f:
            return f.read()

    def write(self, fp, s, mode='w', encoding='utf-8'):
        fp=self._truepath(fp)
        with open(fp, mode=mode, encoding=encoding) as f:
            f.write(s)
    def newfile(self,path,mode='w',encoding='utf-8'):
        path = self._truepath(path)
        open(path,mode=mode,encoding=encoding).close()
    def writefile(self,path,s,mode='w',encoding='utf-8'):
        path = self._truepath(path)
        open(path,mode=mode,encoding=encoding).write(s)
    def save_http_file(self,file,path):
        file.save(self._truepath(path))
    def exists(self,path):
        path = self._truepath(path)
        return os.path.exists(path)
    def isdir(self,path):
        path = self._truepath(path)
        return os.path.isdir(path)
    def isfile(self,path):
        path = self._truepath(path)
        return os.path.isfile(path)
    def islink(self,path):
        path = self._truepath(path)
        return os.path.islink(path)
    def ismount(self,path):
        path = self._truepath(path)
        return os.path.ismount(path)
    def dirname(self,path):
        return os.path.dirname(path)
    def basename(self,path):
        return os.path.basename(path)
    def iterfiles(self,path='/'):
        names = self.listdir(path)
        files=[]
        for name in names:
            if self.isfile(name):
                files.append(name)
        pathes = [self.path + '/' + name for name in files]
        return pathes
    def iterpath(self,path='/'):
        names=self.listdir(path)
        pathes=[self.path+'/'+name for name in names]
        return pathes
    def listdir(self,path='/'):
        path=self._truepath(path)
        return os.listdir(path)
    def rmfile(self,path):
        path=self._truepath(path)
        return os.remove(path)
    def remove(self,path):
        if self.isdir(path):
            return self.rmtree(path)
        if self.isfile(path):
            return self.rmfile(path)
    def rmself(self):
        shutil.rmtree(self.path)
    def rmtree(self,path):
        path = self._truepath(path)
        return shutil.rmtree(path)
    def mkdir(self,path):
        path = self._truepath(path)
        return os.mkdir(path)
    def makedirs(self,path):
        path = self._truepath(path)
        return os.makedirs(path)
    def moveto(self,path,overwrite=False):
        if os.path.exists(path):
            if not overwrite:
                raise Exception('%s already exists.'%(path))
            else:
                shutil.rmtree(path)
        # print(self.path,path)
        copy_dir(self.path,path)
    def copy(self,src,dst):
        src = self._truepath(src)
        dst = self._truepath(dst)
        return shutil.copy(src,dst)
    def copydir(self,src,dst):
        src = self._truepath(src)
        dst = self._truepath(dst)
        return copy_dir(src,dst)
    def rename(self,src,dst):
        src = self._truepath(src)
        dst = self._truepath(dst)
        return os.rename(src,dst)
    def move(self,src,dst):
        src = self._truepath(src)
        dst = self._truepath(dst)
        return shutil.move(src,dst)



class FSItem:
    def __repr__(self):
        return '<%s.%s object:%s>'%(self.__module__,self.__class__.__name__,self.path)

class File(FSItem):
    def __init__(self,path,makefile=False):
        self.path=path
        if makefile:
            if not os.path.exists(path):
                open(self.path,'w').close()

class TextFile(File):
    def __init__(self,path,default=''):
        super().__init__(path,makefile=True)
        if self.read()=='':
            self.write(default)
    def read(self):
        with open(self.path,'r',encoding='utf-8') as f:
            return f.read()
    def write(self,s):
        with open(self.path,'w',encoding='utf-8') as f:
            f.write(s)



class Folder(FakeOS,FSItem):
    def __init__(self,path,writable=False):
        self.writable=writable
        if not os.path.exists(path):
            os.makedirs(path)
        else:
            assert os.path.isdir(path)
        path=os.path.abspath(path)
        self.path=path
        FakeOS.__init__(self,path)
    def clean(self):
        for ch in self.listdir():
            if self.isdir(ch):
                self.rmtree(ch)
            else:
                self.remove(ch)
    def eat(self,path,overwrite=False):
        assert os.path.exists(path)
        if os.path.isdir(path):
            copy_dir(path,self.path)
        else:
            copy_file(path,self.path,overwrite=overwrite)


    def parse_path(self,path):
        items=self._split_path(path)
        return items
    def child(self,name):
        assert not '/' in name
        assert not '\\' in name
        path=self.path+'/'+name
        assert os.path.exists(path)
        if os.path.isdir(path):
            return Folder(path)
        elif os.path.isfile(path):
            return File(path)
        else:
            raise Exception('% should be a file or dir'%(path))
    def route_item(self,items):
        if not items:
            return self
        name=items[0]
        items=items[1:]
        if name=='/':
            return self.route_item(items)
        else:
            if name in self.listdir():
                ch=self.child(name)
                if not len(items):
                    return ch
                else:
                    return ch.route_item(items)
            else:
                raise Exception('%s is not in %s'%(name,self.path))
    def __getitem__(self, item):
        items=self.parse_path(item)
        return self.route_item(items)
    def __setitem__(self, path, value):
        tp=self._truepath(path)
        if os.path.exists(tp):
            if self.writable:
                PowerDirPath(tp).rmself()
            else:
                raise Exception('%s already exists.'%(tp))
        if value is None:
            os.makedirs(tp)
            return
        if isinstance(value,str):
            assert os.path.exists(value)
            if os.path.isfile(value):
                value=File(value)
            else:
                assert os.path.isdir(value)
                value=Folder(value)
        if isinstance(value,Folder):
            value.moveto(tp)
        elif isinstance(value,File):
            shutil.copy(value.path,tp)




import os,shutil,glob,json,uuid,time
from ..ioutils import json_dump,json_load
class FileDB:
    def __init__(self,save_dir,remake_dir=False,check_keys=False):
        '''
        info.json:
            {
                "files":{
                    "file_key1":{
                        "save_filename":"nwdqw",
                        "original_filename":"test.csv"
                    },
                    "file_key2":{
                        ...
                    }
                }
            }
        '''
        self.save_dir=save_dir
        self.files_dir=save_dir+'/files'
        self.info_file=save_dir+'/info.json'
        if remake_dir:
            if os.path.exists(save_dir):
                shutil.rmtree(save_dir)
                time.sleep(0.01)
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        if not os.path.exists(self.files_dir):
            os.makedirs(self.files_dir)
        if not os.path.exists(self.info_file):
            json_dump({},self.info_file)
        if check_keys:
            self._check_keys()

    def eat_file(self,file,key=None,overwrite=False):
        assert os.path.exists(file)
        key=key or self._gen_file_key()
        dst = self.files_dir + '/' + key
        if key in self._get_file_dict().keys():
            if not overwrite:
                raise Exception("Key %s already exists but overwrite option is False."%(key))
            else:
                os.remove(dst)
        shutil.copy(file,dst)
        info={
            "original_filename":os.path.basename(file),
            "save_filename":key
        }
        self._add_to_file_dict(key,info)

    def _gen_file_key(self):
        key=uuid.uuid4().hex
        return key
    def _get_file_dict(self):
        info = json_load(self.info_file)
        return info['files']
    def _add_to_file_dict(self,key,info):
        info=json_load(self.info_file)
        info['files'][key]=info
        json_dump(info,self.info_file)
    def _check_keys(self,raise_error=True):
        ''' assume that info.json files_dir exists. '''
        files=self._get_file_dict()
        files_in_json=[info['save_filename'] for info in files.values()]
        files_exists=os.listdir(self.files_dir)
        if not set(files_in_json)==set(files_exists):
            if raise_error:
                raise Exception("Files exist are not the same as the list in info file.")
            else:
                return False
        return True



from .node import *

class BoostrapBadgeBase(Span):
    def __init__(self,text,type):
        super().__init__(_class="badge badge-%s"%(type))
        self.__call__(text)
class Badges:
    @staticmethod
    def primary(text):
        return BoostrapBadgeBase(text,type="primary")
    @staticmethod
    def secondary(text):
        return BoostrapBadgeBase(text,type="secondary")
    @staticmethod
    def success(text):
        return BoostrapBadgeBase(text,type="success")
    @staticmethod
    def danger(text):
        return BoostrapBadgeBase(text,type="danger")

    @staticmethod
    def warning(text):
        return BoostrapBadgeBase(text,type="warning")

    @staticmethod
    def info(text):
        return BoostrapBadgeBase(text,type="info")

    @staticmethod
    def light(text):
        return BoostrapBadgeBase(text,type="light")

    @staticmethod
    def dark(text):
        return BoostrapBadgeBase(text,type="dark")




from flask import jsonify
from flask_cors import CORS
import json
from wpkit.pan import LocalFSHandle,Pan
from wpkit.web.base import MyBlueprint,Application
from wpkit.web.utils import parse_json,parse_json_and_form,request
from wpkit.web.resources import get_template_by_name
from wpkit.basic import Status,StatusError,StatusSuccess,PointDict


class LocalFSServer(MyBlueprint):
    def __init__(self,import_name='__main__',path="./",url_prefix="/fs",nickname="LocalFSServer",*args,**kwargs):
        self.fs = LocalFSHandle(path=path)
        # print(self.add_to_sitemap)
        super().__init__(import_name=import_name,url_prefix=url_prefix,nickname=nickname,*args,**kwargs)
        # print(self.add_to_sitemap)
    def add_handlers(self):
        @self.route('/',methods=['POST','GET'])
        def do_root():
            return get_template_by_name('pan').render()
        @self.route('/cmd',methods=['POST','GET'])
        @parse_json_and_form
        def do_cmd(cmd):
            print("cmd:", cmd)
            try:
                res = self.fs.execute(cmd)
                res = StatusSuccess(data=res)
            except:
                raise
                res = StatusError()
                return res
            return jsonify(res)
        @self.route('/upload',methods=['POST','GET'])
        @parse_json_and_form
        def do_upload(info):
            file=request.files['file']
            if isinstance(info,str):
                info=json.loads(info)
            info=PointDict.from_dict(info)
            path=self.fs.local_path(info.location)
            path=self.fs.local_path(path+'/'+info.filename)
            file.save(path)
            print('path:',path)
            print('file:',file)
            return StatusSuccess(msg="Uploading succeeded.")
        self.config_statics({
            "/files":self.fs.lpath
        })
        def getUrl(location,name):
            # print("getUrlK")
            res='http://%s:%s'%(self.app.host,self.app.port)+ self.url_prefix+'/files/'+self.fs.local_path(location+'/'+name)
            print("res",res)
            return res
        self.fs.add_cmd('getUrl',getUrl)

if __name__ == '__main__':
    fs=LocalFSServer(url_prefix="/fs")
    fs.run()








from flask import request
from wpkit.web import resources,utils
from wpkit import piu,basic
from wpkit.web.base import MyBlueprint

class BlueBoard(MyBlueprint):
    def __init__(self, import_name=None, name='board', datapath='./data/board', url_prefix='/pan', **kwargs):
        super().__init__(name=name, import_name=import_name, url_prefix=url_prefix, **kwargs)
        self.datapath = basic.DirPath(datapath)
        self.db = piu.Piu(path=self.datapath.db)
        @self.route('/')
        def do_board():
            data = self.db.get('board_data', '')
            return  resources.get_template_by_name('board').render(content=data)
        @self.route('/post', methods=['POST'])
        def do_board_post():
            data = request.get_json()
            self.db.add('board_data', data['content'])
            return 'success'





def generate_hash(s,times=1):
    assert times>=1
    import hashlib
    m = hashlib.md5()
    def gen():
        m.update(s.encode('utf-8'))
        return m.hexdigest()[:10]
    for i in range(times):
        data=gen()
    return data


from wpkit.web.applications.welcome import BlueWelcomePage
from wpkit.web.applications.sitemap import BlueSitemap
from wpkit.web.applications.post_and_download import BluePostAndDownload
from wpkit.web.applications.fsserver import LocalFSServer
from wpkit.web.applications.dbserver import DBServer
from wpkit.web.applications.pan import BluePan
from wpkit.web.applications.board import BlueBoard
from wpkit.web.applications.static import BlueStatic
from wpkit.web.applications.blogserver import BlogServer
from wpkit.web.applications.osserver import OSServer
# from wpkit.web.applications.all import BlueBoard,DBServer,LocalFSServer,BlueWelcomePage,BlueSitemap,BluePostAndDownload,BluePan
from wpkit.web.base import Application,MyBlueprint
from wpkit.pkg_info import is_linux
from wpkit.web.resources import pkg_document_path
class DemoApp(Application):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.register_blueprint(BlueBoard(url_prefix='/board'))
        self.register_blueprint(DBServer(url_prefix='/db'))
        self.register_blueprint(LocalFSServer(url_prefix='/fs',path="./"))
        self.register_blueprint(BlueWelcomePage(url_prefix='/'))
        self.register_blueprint(BluePan(url_prefix="/pan"))
        self.register_blueprint(OSServer(url_prefix="/os",nickname='OS'))
        self.register_blueprint(BlogServer(url_prefix="/blogs",default_root_path='./data/blogs'))
        self.register_blueprint(BlogServer(url_prefix="/documents",nickname='Documentation',default_root_path=pkg_document_path))
        self.sitemap['Blogs']='/blogs/view='
        self.register_blueprint(BlueSitemap(url_prefix='/sitemap'))

        if is_linux():
            self.register_blueprint(BluePostAndDownload(url_prefix='/post_and_download'))

def demo(port=80,host=None,import_name=None):
    from wpkit.linux import get_local_ip,clean_port
    clean_port(port)
    host=host or get_local_ip()
    app = DemoApp(import_name or "__main__")
    print(app.url_map)
    app.run(port=port,host=host)
if __name__ == '__main__':
    import fire
    fire.Fire(demo)




from .GitRepo import default_remote_location,GitRepo,is_git_dir
from wk import Folder,copy_file,copy_dir,copy_fsitem,remove_fsitem,is_empty_dir
from wk.basic import T,TMetaClass,CONST_TYPE
import os,shutil,glob,uuid,random
from .utils import generate_hash
_T=CONST_TYPE
class CONST(metaclass=TMetaClass):
    remote_branch_list=_T()
    master=_T()
    empty=_T()

# TODO: improve performance


_BRANCH_LIST='remote_branch_list'
USER_HOME=os.path.expanduser('~')
STORE_HOME=USER_HOME+'/.store'
BRANCH_LIST_DIR=STORE_HOME+'/BranchLists'


def get_default_path(remote_location,remote_branch):
    name = generate_hash(remote_location + '/' + remote_branch)
    path = BRANCH_LIST_DIR + '/' + name
    return path

def get_default_remote_branch_list(remote_location):
    bd=RemoteBranchList(remote_location=remote_location)
    return bd

class StoreItem(Folder):
    '''
    issue: Branch name has a limit
    '''
    delimiter='.-.'
    special_branches = ['master', 'empty', 'remote_branch_list']
    legal_path_chars = [str(i) for i in range(10)]+[chr(i) for i in range(65, 91)]+[chr(i) for i in range(97, 123)]+list('._-')
    def status(self,repo=None):
        repo=repo or self.repo
        from wk.basic import PointDict
        info=PointDict(
            current_branch=repo.active_branch(),
            local_branches=repo.branch_list(),
            status=repo.status()
        )
        print(info)
        return info
    def __init__(self,path,remote_location=None,remote_branch=None,is_remote_branch_list=False):
        remote_location = remote_location or default_remote_location
        assert remote_branch
        if not os.path.exists(path):
            os.makedirs(path)
        path=os.path.abspath(path)
        Folder.__init__(self,path)
        if is_git_dir(path):
            repo=GitRepo(path)
        else:
            repo=GitRepo.init(path)
        self.repo=repo
        self.path=path
        self.remote_location=remote_location or default_remote_location
        self.remote_branch=remote_branch
        self.data_list=['.git','.type.store'] # clean except
        self.info_list=['.git','.type.store','.more.store']  # copy except
        self.typefile=self.openFiledict('.type.store')
        self.is_remote_branch_list = is_remote_branch_list
        self.init_branches()
        if  not is_remote_branch_list:
            self.rbl=get_default_remote_branch_list(remote_location=self.remote_location)
        print("Init StoreFolder finished.")
    def _try_pull_remote(self):
        try:
            self._pull_remote()
        except:
            import logging
            logging.warning("Can't pull remote branch %s , maybe because local branch has already been updates."%(self.remote_branch))
    def _pull_remote(self):
        repo=self.repo
        repo.pull(self.remote_location,self.remote_branch)
    def _push_self(self):
        repo=self.repo
        repo.add_all()
        repo.commit()
        repo.push(self.remote_location,self.remote_branch)
    def _push_if_not_exists(self):
        if not self.rbl.branch_exists(self.remote_branch):
            self._push_self()
            self.rbl.branch_add(self.remote_branch)
    def _pull_else_push_self(self):
        remote_branch=self.remote_branch
        if not self.rbl.exists(remote_branch):
            self._push_self()
        else:
            self._try_pull_remote()


    # def _pull_remote_branch_list(self,repo=None,remote_location=None,remote_branch='remote_branch_list',hard=False):
    #     repo=repo or self.repo
    #     remote_location=remote_location or self.remote_location
    #     pull=False
    #     if not 'remote_branch_list' in repo.branch_list():
    #         repo.branch_create('remote_branch_list')
    #         pull=True
    #     if hard:
    #         pull=True
    #     if pull:
    #         try:
    #             br = repo.active_branch()
    #             repo.checkout_branch('remote_branch_list')
    #             repo.clean()
    #             repo.add_all()
    #             repo.commit()
    #             repo.pull(remote_location, branch=remote_branch)
    #             repo.checkout_branch(br)
    #         except:
    #             print("Can't pull remote_branch_list, maybe because local branch is already updated.")

    def init_branches(self,repo=None):
        '''
        A store repo has 3 branches: master , empty , remote_branch_list, remote_branch
        '''
        repo=repo or self.repo
        if not repo.branch_list():
            repo.commit() # create master
        if not 'empty' in repo.branch_list():
            repo.branch_create('empty')
            repo.checkout_branch('empty')
            repo.clean()
            repo.commit()
            repo.checkout_branch('master')
        if not self.remote_branch in repo.branch_list():
            repo.branch_create(self.remote_branch)
            repo.checkout_branch(self.remote_branch)
            repo.clean()
            repo.commit()
            repo.checkout_branch('master')
        repo.checkout_branch(self.remote_branch)

    # def _read_remote_branch_list(self,pull=False):
    #     repo=self.repo
    #     br = repo.active_branch()
    #     repo.checkout_branch(CONST.remote_branch_list)
    #     if pull:
    #         self._pull_remote_branch_list(repo)
    #     lf = self.openSimpleListfile(CONST.remote_branch_list)
    #     li = lf.read()
    #     repo.checkout_branch(br)
    #     return li
    # def _add_to_remote_branch_list(self,branch):
    #     repo=self.repo
    #     br=repo.active_branch()
    #     self._pull_remote_branch_list(hard=True)
    #     repo.checkout_branch(CONST.remote_branch_list)
    #     # repo.pull(self.remote_location,CONST.remote_branch_list)
    #     lf=self.openSimpleListfile(CONST.remote_branch_list)
    #     li=lf.read()
    #     # print("original:",li)
    #     li.append(branch)
    #     li=list(set(li))
    #     # print("now:",li)
    #     lf.write(li)
    #     repo.add_all()
    #     repo.commit()
    #     repo.push(self.remote_location,CONST.remote_branch_list)
    #     repo.checkout_branch(br)
    def iter_contentpath(self):
        lis=[]
        for name in self.listdir():
            if name in self.info_list:
                continue
            else:
                path=self.path+'/'+name
                lis.append(path)
        return lis
    def set_type(self,type):
        self.typefile.type=type
        return type
    def get_type(self):
        if not self.typefile.get('type'):
            return None
        return self.typefile.type
    @classmethod
    def pull(cls,remote_location=None,remote_branch=None,path=None,overwrite=False):
        remote_location=remote_location or default_remote_location
        remote_branch=remote_branch or 'master'
        if os.path.exists(path) and len(os.listdir(path)):
            if overwrite:
                shutil.rmtree(path)
            else:
                raise FileExistsError("Can't pull because folder %s is not empty."%(path))
        if not os.path.exists(path):
            os.makedirs(path)
        repo=GitRepo.init(path)
        if not repo.branch_list():
            repo.add_all()
            repo.commit()
        if not remote_branch in repo.branch_list():
            repo.branch_create(remote_branch)
            repo.checkout_branch(remote_branch)
            repo.clean()
        repo.pull(remote_location,branch=remote_branch)
        item=cls(repo.path,remote_location=remote_location,remote_branch=remote_branch)
        type=item.get_type()
        if not type:
            type=item.set_type(T.FOLDER)
            import logging
            logging.warning('StoreItem %s has no type, so we set it as %s'%(item.path,type))
        if type==T.FOLDER:
            return StoreFolder(repo.path,remote_location=remote_location,remote_branch=remote_branch)
        else:
            assert type==T.FILE
            return StoreFile(repo.path,remote_location=remote_location,remote_branch=remote_branch)
    @classmethod
    def openStorefolder(cls,path,remote_location=None,remote_branch=None,force_pull=False,overwrite=False):
        remote_location=remote_location or default_remote_location
        if not is_git_dir(path):
            force_pull=True
        if not force_pull:
            item=StoreFolder(path,remote_location=remote_location,remote_branch=remote_branch)
        else:
            item=StoreFolder.pull(remote_location=remote_location,remote_branch=remote_branch,path=path,overwrite=overwrite)
        return item
    @classmethod
    def openStorefile(cls,path,remote_location=None,remote_branch=None,force_pull=False,overwrite=False):
        remote_location=remote_location or default_remote_location
        if not is_git_dir(path):
            force_pull=True
        if not force_pull:
            item=StoreFile(path,remote_location=remote_location,remote_branch=remote_branch)
        else:
            item=StoreFile.pull(remote_location=remote_location,remote_branch=remote_branch,path=path,overwrite=overwrite)
        return item
    def upload(self,remote_location=None,remote_branch=None,overwrite=True):
        # Todo:get remote branch list
        # deprecated !!!

        remote_loacation=remote_location or self.remote_location
        remote_branch=remote_branch or self.remote_branch
        assert remote_loacation and remote_branch
        assert remote_branch !='master'
        repo=self.repo
        repo.add_all()
        repo.commit()
        # br=repo.active_branch()
        if not remote_branch in repo.branch_list():
            repo.branch_create(remote_branch)
        repo.checkout_branch(remote_branch)

        repo.push(remote_loacation,remote_branch)
        # print("list:", self.listdir())
        # repo.checkout_branch(br)
        self.rbl.branch_add_if_not_exists(remote_branch)
    @classmethod
    def export(cls,path,remote_branch,remote_location=default_remote_location,name=None,cache_dir='.tmp',overwrite=False):
        if os.path.exists(cache_dir):
            shutil.rmtree(cache_dir)
        def _export_dir(obj,path,cache_dir):
            for p in obj.iter_contentpath():
                copy_fsitem(p, path)
            more = obj.morefile.copy()
            # obj.rmself()
            for nm, br in more.items():
                br_cache_dir=cache_dir+'/'+br
                cls.export(path, remote_location=remote_location, remote_branch=br, name=nm, cache_dir=br_cache_dir,overwrite=overwrite)
        this_dir=cache_dir+'/.this'
        obj=StoreItem.pull(remote_location=remote_location,remote_branch=remote_branch,path=this_dir)
        # print(obj.path)
        # print("list:", obj.morefile)
        # print("list:", obj.listdir())
        # input()

        if isinstance(obj,StoreFolder):
            if not os.path.exists(path):
                os.makedirs(path)
            assert os.path.isdir(path)
            name = name or remote_branch.split(cls.delimiter)[-1]
            path=path+'/'+name
            if os.path.exists(path):
                if overwrite:
                    shutil.rmtree(path)
                else:
                    raise Exception("Can't export to %s because path already existed and overwrite is not True")
            os.mkdir(path)
            _export_dir(obj,path,cache_dir)
        else:
            assert isinstance(obj,StoreFile)
            if os.path.exists(path):
                assert os.path.isdir(path)
                if name:
                    path=path+'/'+name
                ps=obj.iter_contentpath()
                ps.sort()
                p=ps[0]
                # path=path+'/'+os.path.basename(p)
                copy_fsitem(p, path)

            else:
                for p in obj.iter_contentpath():
                    copy_fsitem(p, path)
            # obj.rmself()
        # remove_fsitem(cache_dir)
        # shutil.rmtree(cache_dir)
    @classmethod
    def uploadStoreitem(cls,path, remote_location, remote_branch, cache_dir,add_more=None):
        assert os.path.exists(path)
        if os.path.isdir(path):
            tmp = StoreFolder(cache_dir, remote_location=remote_location, remote_branch=remote_branch)
        else:
            tmp = StoreFile(cache_dir, remote_location=remote_location, remote_branch=remote_branch)
        tmp.clean()
        if os.path.isfile(path):
            tmp.eat(path)
        else:
            if add_more:
                for k,v in add_more.items():
                    tmp.addmore(k,v)
            for p in os.listdir(path):
                p = path + '/' + p
                tmp.eat(p)
        tmp.upload(remote_location=remote_location, remote_branch=remote_branch)
    @staticmethod
    def is_legal_path_to_upload(path):
        path=os.path.basename(path)
        legal_path_chars=StoreItem.legal_path_chars
        # print(legal_path_chars)
        if StoreItem.delimiter in path:
            import logging
            logging.warning('Illegal path "%s"!' % (path))
            return False
        for ch in path:
            if ch not in legal_path_chars:
                import logging
                logging.warning('Illegal path "%s"!'%(path))
                return False
        return True

    @classmethod
    def uploadStoreitemRecursive(cls,path, remote_location=None, remote_branch=None,
                                 cache_dir='.store.upload.cache',depth=-1,check_path=True):
        # todo: check branch name
        # assert remote_branch not in cls.special_branches
        assert depth>=0 or depth==-1
        assert os.path.exists(path)
        if check_path:
            assert cls.is_legal_path_to_upload(path)
        remote_location=remote_location or default_remote_location
        assert remote_branch
        if os.path.exists(cache_dir):
            shutil.rmtree(cache_dir)
        target_dir=cache_dir+'/target'
        store_dir=cache_dir+'/stores'
        os.makedirs(target_dir)
        copy_fsitem(path,target_dir)
        path=target_dir+'/'+os.path.basename(path)
        return cls._uploadStoreitemRecursive(path, remote_location, remote_branch, cache_dir=store_dir,depth=depth,check_path=check_path)
    @classmethod
    def _uploadStoreitemRecursive(cls,path, remote_location, remote_branch, cache_dir,depth=0,check_path=True):
        assert os.path.exists(path)
        if check_path:
            assert cls.is_legal_path_to_upload(path)
        if os.path.exists(cache_dir):
            shutil.rmtree(cache_dir)
        os.makedirs(cache_dir)
        print('path:',path)
        if depth==0:
            if os.path.isdir(path):

                tmp = StoreFolder(cache_dir, remote_location=remote_location, remote_branch=remote_branch)
                tmp.clean()
                for p in os.listdir(path):
                    p = path + '/' + p
                    tmp.eat(p)
            else:
                tmp = StoreFile(cache_dir, remote_location=remote_location, remote_branch=remote_branch)
                tmp.clean()
                tmp.eat(path)
        else:
            import uuid
            if os.path.isdir(path):
                self_cache_dir = cache_dir+'/self-cache-' + uuid.uuid4().hex
                more={}
                for name in os.listdir(path):
                    p=path+"/"+name
                    if check_path:
                        assert cls.is_legal_path_to_upload(p)
                    item_cache_dir=cache_dir+'/item-cache-'+name
                    os.mkdir(item_cache_dir)
                    item_branch=remote_branch+cls.delimiter+name
                    cls._uploadStoreitemRecursive(path=p,remote_location=remote_location,remote_branch=item_branch,cache_dir=item_cache_dir,depth=depth-1)
                    more[name]=item_branch
                os.mkdir(self_cache_dir)
                tmp = StoreFolder(self_cache_dir, remote_location=remote_location, remote_branch=remote_branch)
                tmp.morefile.update(more)
            else:
                tmp = StoreFile(cache_dir, remote_location=remote_location, remote_branch=remote_branch)
                tmp.clean()
                tmp.eat(path)
        tmp.upload(remote_location=remote_location, remote_branch=remote_branch)
        remove_fsitem(path)
    def is_empty(self):
        names=self.listdir()
        for name in names:
            if name not in self.data_list:
                return False
        return True
    def clean(self):
        names=self.listdir()
        for name in names:
            if name in self.data_list:
                continue
            self.remove(name)
            if name=='.more.store':
                self.openFiledict(name)

class StoreFolder(StoreItem):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.morefile = self.openFiledict('.more.store')
        self.set_type(T.FOLDER)
    def addmore(self,name,branch):
        self.morefile[name]=branch
    def eatStore(self,path,name=None,remote_location=None,remote_branch=None,upload=True,overwrite=False,cache_dir='.tmp',in_depth=0):
        if os.path.exists(cache_dir):
            shutil.rmtree(cache_dir)
        os.makedirs(cache_dir)
        assert os.path.exists(path)
        if not name:
            name=os.path.basename(os.path.abspath(path))
        remote_location = remote_location or self.remote_location
        assert remote_location
        if not remote_branch:
            assert self.remote_branch
            remote_branch=self.remote_branch+self.delimiter+name
        if upload:
            StoreItem.uploadStoreitem(path,remote_location=remote_location,remote_branch=remote_branch,cache_dir=cache_dir)
        self.morefile[name]=remote_branch

class StoreFile(StoreItem):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.set_type(T.FILE)



class RemoteBranchList(StoreFolder):
    def __init__(self,path=None,remote_location=None,remote_branch=None):
        remote_branch=remote_branch or _BRANCH_LIST
        if not path:
            path=get_default_path(remote_location,remote_branch)
        super().__init__(path,remote_location,remote_branch,is_remote_branch_list=True) # create local branch
        self._init_remote_branch_list()
    def _init_remote_branch_list(self):
        repo=self.repo
        try:
            self._pull_remote()
            if self.is_empty() or not self.exists(_BRANCH_LIST):
                self.openSimpleListfile(_BRANCH_LIST)
                self._push_self()
        except:
            import logging
            logging.warning("Can't pull from remote, maybe because local branch is already updated , or remote branch doesn't exist.")
    def sync_list(self):
        self._try_pull_remote()
    def read_list(self):
        return self.list_branches()
    def list_branches(self):
        return self._read_remote_branch_list()
    def branch_add_if_not_exists(self,branch):
        return self.branch_add(branch)
    def branch_add(self,branch):
        lf=self.openSimpleListfile(_BRANCH_LIST)
        lis=lf.read()
        if branch not in lis:
            self._try_pull_remote()
            lis.append(branch)
            lf.write(lis)
            self._push_self()
    def branch_exists(self,branch):
        if branch in self._read_remote_branch_list():
            return True
        else:
            self._try_pull_remote()
            if branch in self._read_remote_branch_list():
                return True
            else:
                return False
    def _read_remote_branch_list(self):
        return self.openSimpleListfile(_BRANCH_LIST).read()







from wpkit.web.base import  MyBlueprint
from wpkit.web import utils,resources
class BlueSitemap(MyBlueprint):
    def __init__(self,import_name=None,name='sitemap',sitemap={},url_prefix='/sitemap',**kwargs):
        super().__init__(name=name,import_name=import_name,url_prefix=url_prefix,add_to_sitemap=False,**kwargs)
        @self.route('/')
        def do_root():
            temf = resources.default_templates['sitemap']
            return utils.render(open(temf, 'r', encoding='utf-8').read(), sitemap=self.get_sitemap())
    def register(self, app,*args,**kwargs):
        def get_sitemap():
            return app.sitemap
        self.get_sitemap=get_sitemap
        return MyBlueprint.register(self,app,*args,**kwargs)


import os,shutil,glob,json
from .. ioutils import json_dump,json_load,read_txt,write_txt,read_config,write_config

class ObjectFile:
    def __init__(self,path,default=None,recreate=False):
        self.path=path
        if recreate and os.path.exists(path):
            os.remove(path)
        if not os.path.exists(path):
            open(path,'w').close()
        if self.is_empty():
            self.write(default)
    def is_empty(self):
        with open(self.path) as f:
            s=f.read()
            if s=='':
                return True
        return False
    def __call__(self, obj=None):
        if obj is None:
            return self.read()
        return self.write(obj)
    def read(self):
        return json_load(self.path)
    def write(self,obj):
        return json_dump(obj, self.path)
class SimpleListFile(ObjectFile):
    def __init__(self,path,split_char='\n'):
        self.path=path
        self.split_char=split_char
        super().__init__(self.path,default=[])
    def write(self,obj):
        obj=self.split_char.join(obj)
        return write_txt(obj,self.path)
    def append(self,obj):
        lis=self.read()
        lis.append(obj)
        return self.write(lis)
    def read(self):
        text=read_txt(self.path).strip()
        return text.split(self.split_char)
class DictFile(dict):
    def __init__(self,path):
        self.path=path
        if path.exists():
            assert os.path.isfile(path)
            dic=json_load(path)
            assert isinstance(dic,dict)
        else:
            dic={}
            open(path,'w').close()
            json_dump(dic,path,indent=4)
        super().__init__(dic)
    def __setattr__(self, key, value):
        dict.__setattr__(self,key,value)
        self._save()
    def __setitem__(self, key, value):
        dict.__setitem__(self,key,value)
        self._save()
    def update(self,*args,**kwargs):
        for k,v in kwargs.items():
            self[k]=v
        for arg in args:
            self.update(**arg)
    def _save(self):
        json_dump(self,self.path,indent=4)

class SimpleConfigFile(dict):
    def __init__(self,path,splitchar='=',comment_tag='#',sync=True):
        self.path=path
        self.splitchar=splitchar
        self.sync=sync
        super().__init__(read_config(path,splitchar=splitchar,comment_tag=comment_tag))
    def __setitem__(self, key, value):
        res=dict.__setitem__(self,key,value)
        if self.sync:
            self._save()
        return res
    def save(self):
        return self._save()
    def _save(self):
        return self.write()
    def write(self):
        write_config(self,self.path,splitchar=self.splitchar)

class NumberFile(ObjectFile):
    def __init__(self,path,default=0):
        super().__init__(path,default)



from wk.extra.gitspace import Store

import fire
class Cli:

    @classmethod
    def download(cls,key,path=None,overwrite=False):
        from wk.extra.gitspace import Store
        store = Store()
        store.get(key,path=path,overwrite=overwrite)
    @classmethod
    def upload(cls,key,path,recursive=False):
        from wk.extra.gitspace import Store
        store = Store()
        store.set(key,path,recursive)
if __name__ == '__main__':
    fire.Fire(Cli)


import shutil,os,glob
def setup_default(out_dir='./mysite',add_as_service=True):
    dir=os.path.dirname(__file__)+'/pjfiles'
    if not os.path.exists(out_dir):os.makedirs(out_dir)
    for i,f in enumerate(glob.glob(dir+'/*')):
        if os.path.isdir(f):continue
        f2=out_dir+'/'+os.path.basename(f)
        shutil.copy(f,f2)
    if not add_as_service:return  True
    from wk.extra.linux.tools import add_py_to_service
    add_py_to_service(service_name="mysite",project_dir=out_dir,script_file_path=out_dir+'/main.py')




from wpkit.web.base import MyBlueprint
from wpkit.web import utils,resources

class BlueWelcomePage(MyBlueprint):
    # add_to_sitemap = True
    def __init__(self,import_name=None,name='welcome',url_prefix='/welcome',**kwargs):
        super().__init__(name=name,import_name=import_name,url_prefix=url_prefix,add_to_sitemap=False,**kwargs)
        @self.route('/')
        def do_root():
            return resources.get_template_by_name('welcome.tem').render(links=self.app.sitemap)
    def register(self, app, *args,**kwargs):
        self.app=app
        super().register(app,*args,**kwargs)


from wk.web import Application,MyBlueprint,request,parse_args
from wk.extra import node
from wk import join_path,FakeOS
from . import pages
import jieba

class FileServer(MyBlueprint):
    url_prefix = '/'
    def __init__(self,import_name,serve_root,*args,**kwargs):
        super().__init__(import_name,*args,**kwargs)
        self.serve_root=serve_root
        self.files_location='/files'
        self.file_engine=FakeOS(self.serve_root)
        self.add_static('/files',self.serve_root)


        @self.route('/', methods=['get'])
        def do_root():
            return pages.SearchPageBase().render()

        @self.route('/search', methods=['post', 'get'])
        @parse_args
        def do_search(search_keywords):
            keywords=[]
            for word in search_keywords.split():
                keywords+=list(jieba.cut(word))
            # print(keywords)
            result = self.file_engine.search(keywords)
            # print(result)
            result= [self.file_info(file) for file in result]

            result=[
                pages.FileCard(x) for x in result
            ] or "无结果"
            x = pages.SearchPageBase().compile(search_result=result).render(input_fill=search_keywords)
            return x
    def file_info(self,file):
        url=join_path(self.url_prefix,self.files_location,file)
        info=self.file_engine.info(file)
        info.update(url=url)
        return info



from .StoreItem import *
from .utils import generate_hash

_BRANCH_LIST='remote_branch_list'
USER_HOME=os.path.expanduser('~')
STORE_HOME=USER_HOME+'/.store'
BRANCH_LIST_DIR=STORE_HOME+'/BranchLists'
SHADOW_STORE_HOME=STORE_HOME+'/ShadowStores'
def get_default_shadowstore_path():
    return SHADOW_STORE_HOME+'/main'

class ShadowStore:
    def __init__(self,path=None,remote_location=None,sync_keys=False):
        remote_location=remote_location or default_remote_location
        path=path or get_default_shadowstore_path()
        self.path=path
        self.remote_location=remote_location
        print("Creating empty branch...")
        # self.folder=StoreFolder.openStorefolder(self.path,remote_location=self.remote_location,remote_branch='empty')
        self.rbl=get_default_remote_branch_list(remote_location=self.remote_location)
        print("Create empty branch successful")
        if sync_keys:
            self.sync_keys()
        print("Init shadow store finish.")
    def sync_keys(self):
        # self.folder.rbl.sync_list()
        self.rbl.sync_list()
        return self.keys()
    def keys(self):
        # return self.folder.rbl.read_list()
        return self.rbl.read_list()
    def is_legal_key(self,key):
        legal_chars=StoreItem.legal_path_chars+['/']
        if StoreItem.delimiter in key:
            return False
        for ch in key:
            if not ch in legal_chars:
                return False
        return True
    def key_to_branch(self,key):
        assert self.is_legal_key(key)
        remote_branch = key.replace('/', StoreItem.delimiter)
        return remote_branch
    def get(self,key,path=None,overwrite=False):
        path=path or './'
        if os.path.exists(path):
            if os.path.isfile(path):
                if overwrite:
                    remove_fsitem(path)
                else:
                    raise FileExistsError('File %s already exists.'%(path))
            if os.path.isdir(path):
                tp=path+'/'+os.path.basename(key)
                if os.path.exists(tp):
                    if (os.path.isdir(tp) and not is_empty_dir(tp)) or os.path.isfile(tp):
                        if overwrite:
                            remove_fsitem(tp)
                        else:
                            raise FileExistsError('File %s already exists.' % (tp))
        remote_branch=self.key_to_branch(key)
        # print(remote_branch,self.keys())
        assert remote_branch in self.keys()
        print("exporting... %s,%s"%(path,remote_branch))
        StoreItem.export(path,remote_location=self.remote_location,remote_branch=remote_branch,cache_dir=self.path)
        return True
    def set(self,key,path,recursive=False,add_more=None):
        remote_branch=self.key_to_branch(key)
        if not recursive:
            StoreItem.uploadStoreitem(path,remote_location=self.remote_location,remote_branch=remote_branch,cache_dir=self.path,add_more=add_more)
        else:
            StoreItem.uploadStoreitemRecursive(path,remote_location=self.remote_location,remote_branch=remote_branch,cache_dir=self.cache_dir)




from wpkit.web.base import  MyBlueprint
from wpkit.web.resources import get_env,get_book
from wpkit.web import resources
from wpkit.piu import Piu
from wpkit.basic import Path,DirPath,standard_path,PowerDirPath,get_relative_path,load_config
from flask import send_file
import os



class BlogServer(MyBlueprint):
    add_to_sitemap = True
    def __init__(self,url_prefix='/blogs',default_root_path='data/blogs',data_path="data/blogs",nickname='Blogs', *args,**kwargs):
        self.data_path=Path(data_path)
        self.db_path=self.data_path/'db'
        self.db=Piu(path=self.db_path)
        default_root_path=os.path.abspath(default_root_path)
        self.db.set('root_path',default_root_path)
        super().__init__(url_prefix=url_prefix, nickname=nickname, *args, **kwargs)
    def get_visit_link(self):
        def get_link(path):
            path+='.'
            parts=path.split('/')
            parts[-1]='view='+parts[-1][:-1]
            return '/'.join(parts)
        return standard_path(self.url_prefix)+'/view='
    def add_handlers(self):
        @self.route('/', defaults={'req_path': ''})
        @self.route('/<path:req_path>')
        def do_route(req_path):
            print('reqpath:',req_path)
            root_path=self.db.get('root_path')
            print("root_path:",root_path)
            real_path=root_path+'/'+req_path if req_path!='' else root_path
            real_path=PowerDirPath(real_path)
            print("real_path:",real_path)
            basename=real_path.basename()
            if basename.startswith('view='):
                real_path=real_path.dirname()/basename[len('view='):]
                print("real_path:",real_path)
                if real_path.isdir():
                    return self.do_view_dir(real_path)
                else:
                    return self.do_view_file(real_path)
            else:
                if real_path.isdir():
                    return self.do_send_dir(real_path)
                elif real_path.isfile():
                    return self.do_send_file(real_path)
            return 'not finished.'
    def do_send_file(self,path):
        return send_file(path)
    def do_send_dir(self,path):
        rel_path,names,urls,_=self.parse_dir_path(path)
        items = dict(zip(names, urls))
        return resources.Pages.links.render(links=items)
    def do_view_file(self,path):
        # print("path:",path)
        if path.endswith('.md'):
            tem=self.get_template('view_md.tem',os.path.dirname(path))
            # print(tem)
            return tem.render(markdown_data=PowerDirPath(path)())
        elif path.endswith('.page'):
            tem=self.get_template(os.path.basename(path),os.path.dirname(path))
            dir = path if os.path.isdir(path) else os.path.dirname(path)
            fs = os.listdir(dir)
            map = dict({f: f for f in fs})
            return tem.render(map=map)
        elif path.endswith('.book'):
            return get_book(path)
        elif path.endswith('.txt') or \
                path.endswith('.json') or \
                path.endswith('.bat') or \
                path.endswith('.sh') or \
                path.endswith('.js') or \
                path.endswith('.css') or \
                path.endswith('.py'):
            tem=self.get_template('view_file.tem',os.path.dirname(path))
            return tem.render(text_data=PowerDirPath(path)())
        else:
            return self.do_send_file(path)
    def do_view_dir(self,path):
        rel_path,names,_,urls=self.parse_dir_path(path)
        items = dict(zip(names, urls))
        print('items:', items)

        if 'index.html' in names:
            return send_file(path+'/'+'index.html')
        if "index.tem" in names:
            tem=self.get_template("index.tem",path)
            return  tem.render(links=items)
        elif "index.md" in names:
            return self.do_view_file(path+'/index.md')
            # tem=self.get_template('sys/index.tem',path)
            # return tem.render(body=PowerDirPath(path+'/index.md')())
        return resources.Pages.links.render(links=items)
    def get_template(self,name,dir=None):
        env=get_env(path=dir)
        try:
            tem=env.get_template(name)
        except:
            print("template not found:",name)
            return None
        return tem
    def parse_dir_path(self,path):
        root_path = Path(self.db.get('root_path'))
        root_path=standard_path(root_path)
        path=standard_path(path)
        rel_path = get_relative_path(root_path,path)
        rel_path_url=self.get_url(rel_path)
        rel_dir = os.path.dirname(rel_path)
        rel_dir_url = self.get_url(rel_dir)
        path = DirPath(path)
        names = path.list()
        vnames = ['view=' + i for i in names]
        view_urls = [standard_path(rel_path_url+'/' + vname) for vname in vnames]
        urls=[standard_path(rel_path_url+'/'+name) for name in names]
        return rel_path,names,urls,view_urls


if __name__ == '__main__':
    BlogServer(url_prefix='/').run()









import jieba
from wk import string_utils as TU
def get_keywords(text):
    keywords = list(jieba.cut_for_search(text))
    keywords = list(filter(lambda word: TU.is_all_chinese(word) or TU.is_all_alphabet_or_digit(word), keywords))
    keywords = list(set(keywords))
    return keywords


import os,shutil,glob,inspect
class _T(str):
    def __call__(self, s):
        if s.upper()==self or s==self:
            return True
        else:
            return False
CONST_TYPE=_T
class TMetaClass(type):
    def __new__(cls, name, bases, attrs):
        dic=attrs.copy()
        for k,v in attrs.items():
            if isinstance(v,_T):
                dic[k]=_T(k)
        return type.__new__(cls, name, bases, dic)

class T(metaclass=TMetaClass):
    NOT_FOUND=_T()
    NOT_EXISTS=_T()
    NO_VALUE=_T()
    NOT_IMPLEMENTED=_T()
    NOT_ALLOWED=_T()
    EMPTY=_T()
    NO_SUCH_VALUE=_T()
    NO_SUCH_ATTR=_T()
    NOT_GIVEN=_T()
    FOLDER=_T()
    FILE=_T()
    DIR=_T()
    LINK=_T()
    MOUNT=_T()
    IMAGE=_T()
    TEXT=_T()
    VIDEO=_T()
    AUDIO=_T()
    JSON=_T()
    TXT=_T()
    PY=_T()
    JPG=_T()
    PNG=_T()
    JPEG=_T()
    GIF=_T()
    PGM=_T()
    BMP=_T()
    JS=_T()
    MP4=_T()
    AVI=_T()
    IMAGE_FILE_EXTS=[JPG,JPEG,PNG,GIF,PGM,BMP]


class EnumTypeMetaClass(type):
    def __new__(cls, name, bases, attrs):
        dic=attrs.copy()
        for k,v in attrs.items():
            if isinstance(v,_T):
                dic[k]=v
            else:
                dic[k]=_T(k)
        return type.__new__(cls, name, bases, dic)
class WT(metaclass=EnumTypeMetaClass):
    Error=0
def get_file_ext(path):
    name=os.path.basename(path)
    if not '.' in path:return None
    ext=name.rsplit('.',maxsplit=1)[-1]
    if ext=='':ext=None
    return ext
def read_file(path,type=None,encoding=None):
    assert os.path.isfile(path)
    type=type or get_file_ext(path) or 'TXT'
    if type.upper() in T.IMAGE_FILE_EXTS:
        from PIL import Image
        return Image.open(path)
    else:
        if not encoding:
            encoding='utf-8'
        with open(path,'r',encoding=encoding) as f:
            return f.read()

class IterObject(dict):
    __no_value__='<__no_value__>'
    def __getattr__(self, key):
        v=self.get(key,self.__no_value__)
        if v is self.__no_value__:
            self[key]=IterObject()
            return self[key]
        else:
            return v
    def __setattr__(self, key, value):
        self[key]=value

class Path(str):
    __no_value__ = '<__no_value__>'
    def __init__(self,*args,**kwargs):
        super().__init__()
    def __getattr__(self, item):
        return self/item
    def __setattr__(self, key, value):
        self.__dict__[key]=value
    def __truediv__(self, other):
        return Path(self+'/'+other)
    def __call__(self,s=T.NO_VALUE):
        if T.NO_VALUE:return None
        return self/s
    def relative_path(self,path):
        assert path.startswith(self)
        if self=='':
            pass
        else:
            if path.startswith(self):
                path=path[len(self):]
            path=path.lstrip('/')
        return self.__class__(path)

class StrictPath:
    def __init__(self,s):
        self.__value__=Path(self.__strict__(s))
    def __strict__(self,s):
        prefix='/' if s.startswith('/') or s.startswith("\\") else ''
        def remove_all(lis,item):
            if item in lis:
                lis2=lis.copy()
                for i in lis:
                    if i==item:
                        lis2.remove(i)
                return lis2

            else:
                return lis
        lis=s.split('/')
        lis2=[]
        for i in lis:
            lis2+=i.split('\\')
        lis=lis2
        lis=remove_all(lis,'/')
        lis=remove_all(lis,"\\")
        lis=remove_all(lis,'')
        return prefix+'/'.join(lis)
    def __getattr__(self, item):
        return self/item
    def __truediv__(self, other):
        return StrictPath(self.__value__/other).__value__
    def __call__(self, s=''):
        if s=='':return self.__value__
        return StrictPath(self.__value__/s)
    def __repr__(self):
        return "<StrictPath:'%s'>"%(self.__value__)
    def __str__(self):
        return self.__value__
def join_path(*args):
    return StrictPath('/'.join(args))()
def standard_path(p,check=False):
    assert len(p)
    # if (len(p)<=4 and p[1:]=='://') or (len(p)==3 and p[1:]==':/'):return p[:3]
    # print(p)
    p=str(StrictPath(p))
    # print(p)
    # if not '/' in p:return p
    p=p.split('/')
    assert len(p)
    res=[]
    p.reverse()
    while True:
        if not len(p):
            if len(res)>1 and '.' in res:
                for char in res:
                    if char=='.':
                        res.remove(char)
            path='/'.join(res)
            if len(path)==2 and path[1]==':':
                path=path+'/'
            return path
        i=p.pop()
        if i=='':
            res.append('')
            continue
        elif i=='.':
            if not len(res):res.append('.')
        elif i=='..':
            if check:return False
            if not len(res) or res[-1]=='.':raise Exception('Error,path has reached the top when another ".." shew up.')
            else:
                res.pop()
                if not len(res):res.append('.')
        else:res.append(i)
def split_path(path):
    path=standard_path(path)
    items=path.split('/')
    if items[0]=='':
        items[0]='/'
    return items

def join_standard_path(*args):
    path=join_path(*args)
    return standard_path(path)
def get_relative_path(root,child):
    path=child
    path=standard_path(path)
    root=standard_path(root)
    # print("path:,root:",path,root)
    assert path.startswith(root)
    return path[len(root):]
class SecureDirPath(str):

    __no_value__ = '<__no_value__>'
    def __init__(self,s):
        super().__init__()
    def __getattr__(self, item):
        return self/item
    def __truediv__(self, other):
        return SecureDirPath(StrictPath(self+'/'+other))
    def __call__(self):
        assert os.path.exists(self)
        return self.__read__()
    def file(self,fn):
        fp=self/fn
        return fp
    def __read__(self):
        import os
        if os.path.isfile(self):
            with open(self,'r',encoding='utf-8') as f:
                return f.read()
        if os.path.isdir(self):
            return os.listdir(self)

class DirPath(str):
    __type_file__ = T.FILE
    __type_dir__ = T.DIR
    __type_link__ =T.LINK
    __type_mount__=T.MOUNT
    __type_not_exists__=T.NOT_EXISTS
    __no_value__ = T.NO_VALUE
    # __type_file__ = '<type:file>'
    # __type_dir__ = '<type:dir>'
    # __type_link__ = '<type:link>'
    # __type_mount__='<type:mount>'
    # __type_not_exists__='<type:not_exists>'
    # __no_value__ = '<__no_value__>'
    def __init__(self,s):
        super().__init__()
    def __getattr__(self, item):
        return self/item
    def __truediv__(self, other):
        return DirPath(StrictPath(self+'/'+other))
    def __call__(self, s=__no_value__,*args,**kwargs):
        assert os.path.exists(self)
        if s is self.__no_value__:
            return self.__read__(*args,**kwargs)
        else:
            return self.__write__(s,*args,**kwargs)
    def info(self,format=False,with_path=False):
        assert self.exists()
        info=FSItemInfo()
        info.atime=self.getatime()
        info.ctime=self.getctime()
        info.mtime=self.getmtime()
        info.type=self.type()
        info.name=self.basename()
        info.size=os.path.getsize(self)
        if with_path:
            info.path=self.abspath()
        if format:
            info.pretty_format()
        return info
    def tranverse_info(self,depth=-1,format=False):
        info=self.info(format=format)
        if depth==1:
            return info
        if self.isdir():
            info.children={}
            for item in self.children():
                info.children.update({item.basename():item.tranverse_info(depth=depth-1,format=format)})
        return info
    def standard(self):
        return self.__class__(standard_path(self))
    def tranverse_files(self,func):
        from wk import tranverse_files
        tranverse_files(self,func)
    def tranverse_dirs(self,func):
        from wk import tranverse_dirs
        tranverse_dirs(self,func)
    def deepglob(self,s,res=[]):
        res+=self.glob(s)
        for d in self.dirs():
            res=d.deepglob(s,res)
        return res
    def glob(self,s):
        fs=glob.glob(self.join_path(s))
        fs=[self.__class__(f) for f in fs]
        return fs
    def children(self):
        children=[self.join_path(item) for item in self.listdir()]
        return children
    def files(self):
        items=[]
        for item in self.children():
            if item.isfile():
                items.append(item)
        return items
    def dirs(self):
        items=[]
        for item in self.children():
            if item.isdir():
                items.append(item)
        return items
    def join_path(self,path):
        return self/path
    def list(self):
        assert self.isdir()
        return os.listdir(self)
    def listdir(self):
        return self.list()
    def type(self):
        if self.isfile():return self.__type_file__
        if self.isdir():return self.__type_dir__
        if self.islink():return self.__type_link__
        if self.ismount():return self.__type_mount__
        return self.__type_not_exists__
    def isfile(self):
        return os.path.isfile(self)
    def isdir(self):
        return os.path.isdir(self)
    def islink(self):
        return os.path.islink(self)
    def ismount(self):
        return os.path.ismount(self)
    def isabs(self):
        return os.path.isabs(self)
    def abspath(self):
        return self.__class__(os.path.abspath(self))
    def lexists(self):
        return os.path.lexists(self)
    def exists(self):
        return os.path.exists(self)
    def basename(self):
        return os.path.basename(self)
    def dirname(self):
        name=os.path.dirname(self) if os.path.dirname(self)!='' else '.'
        return self.__class__(name)
    def getatime(self):
        return os.path.getatime(self)
    def getctime(self):
        return os.path.getctime(self)
    def getmtime(self):
        return os.path.getmtime(self)
    def getsize(self):
        assert self.isfile()
        return os.path.getsize(self)
    def add(self,s):
        assert self.isfile()
        with open(self,'a',encoding='utf-8') as f:
            f.write(s)
        return self
    def file(self,fn,encoding='utf-8'):
        fp=self/fn
        if not os.path.exists(fp):
            with open(fp,'w',encoding=encoding) as f:
                pass
        return fp
    def size(self):
        assert self.isfile()
        return os.path.getsize(self)
    def todir(self):
        if not os.path.exists(self):
            os.makedirs(self)
        else:assert self.isdir()
        return self
    def tofile(self):
        if not os.path.exists(self):
            self.dirname().todir().file(self.basename())
        else:
            assert self.isfile()
        return self
    def __write__(self,s,encoding='utf-8',*args,**kwargs):
        assert os.path.isfile(self) or os.path.isdir(self)
        if os.path.isfile(self):
            with open(self,'w',encoding=encoding) as f:
                f.write(s)
                return self
        else:
            s2 = self / s
            os.mkdir(s2) if not os.path.exists(s2) else None
            return s2
    def __read__(self,*args,**kwargs):
        import os
        if os.path.isfile(self):
            return read_file(self)
        if os.path.isdir(self):
            return os.listdir(self)

def get_file_info(path,depth=-1,format=True):
    return DirPath(path).tranverse_info(depth=depth,format=format)
class PowerDirPath(DirPath):
    '''
    This class can be very distructive.
    Be Really Careful !!!
    '''
    def rmself(self):
        assert os.path.isdir(self) or os.path.isfile(self)
        if os.path.isdir(self):
            shutil.rmtree(self)
        else:
            os.remove(self)
        return self

    def __truediv__(self, other):
        return PowerDirPath(DirPath(self).__truediv__(other))

class PointDict(dict):
    __no_value__='<__no_value__>'
    def __getattr__(self, key ,default=T.NOT_GIVEN):
        if key in self.keys():
            return self[key]
        elif default!=T.NOT_GIVEN:
            return default
        raise KeyError('No such key named %s'%(key))
    def __setattr__(self, key, value):
        self[key]=value
    def __call__(self, key , value=__no_value__):
        if value is self.__no_value__:
            self[key]=PointDict()
        else:
            self[key]=value
        return self[key]
    def set_attribute(self,key,value):
        self.__dict__[key] = value
    def get_attribute(self,*args,**kwargs):
        return self.__dict__.get(*args,**kwargs)
    def seta(self,**kwargs):
        for k,v in kwargs.items():
            self.set_attribute('__%s__'%(k),v)
    def geta(self,key,*args,**kwargs):
        return self.get_attribute('__%s__'%(key),*args,**kwargs)
    @classmethod
    def from_dict(cls,dic):
        dic2=cls()
        for k,v in dic.items():
            if not isinstance(v,dict):
                dic2[k]=v
            else:
                dic2[k]=cls.from_dict(v)
        return dic2
    def print(self):
        import json
        print(json.dumps(self,sort_keys=True,indent=4))
    def print1(self,depth=0,step=5,space_around_delimiter=1,fillchar=' ',cell_border='|',delimiter=':'):
        import re
        def len_zh(data):
            temp = re.findall('[^a-zA-Z0-9.]+', data)
            count = 0
            for i in temp:
                count += len(i)
            return (count)
        for k,v in self.items():
            for i in range(depth):
                print(fillchar*step,end='')
                print(cell_border,end='')
            print(k.rjust(step-len_zh(k),fillchar),end=' '*space_around_delimiter+delimiter+' '*space_around_delimiter)
            if not isinstance(v,PointDict):
                print(v)
            else:
                print('\n',end='')
                v.print1(depth=depth+1,step=step,space_around_delimiter=space_around_delimiter,
                          cell_border=cell_border,fillchar=fillchar,delimiter=delimiter)
    def pprint1(self):
        self.print1(step=5, space_around_delimiter=0, fillchar='`', cell_border='|', delimiter=':')

class FSItemInfo(PointDict):
    def pretty_format(self):
        self.atime=to_datetime_str(self.atime)
        self.ctime=to_datetime_str(self.ctime)
        self.mtime=to_datetime_str(self.mtime)
        self.size=pretty_format_size(self.size)
        return self




def dir_tree(dir):
    dic=PointDict()
    items=os.listdir(dir)
    for item in items:
        path=dir+'/'+item
        if os.path.isdir(path):
            dic[item]=dir_tree(path)
        else:
            dic[item]=item
    return dic


def pretty_format_size(size):
    def gen_str(size,type):
        if size%1==0:return '%d %s'%(size,type)
        return '%.2f %s'%(size,type)
    def inrange(s):
        if size>=0 and size <1000:
            return True
    if inrange(size):return gen_str(size,'Bytes')
    size/=1024
    if inrange(size):return gen_str(size,'KB')
    size/=1024
    if inrange(size):return gen_str(size,'MB')
    size/=1024
    if inrange(size):return gen_str(size,'GB')
    size/=1024
    if inrange(size):return gen_str(size,'TB')
    size/=1024
    return gen_str(size,'PB')
def to_datetime_str(t):
    import time
    t=time.gmtime(t)
    return time.strftime('%Y-%m-%d %H:%M:%S',t)
def to_date_str(t):
    import time
    t=time.gmtime(t)
    return time.strftime('%Y-%m-%d',t)

class FileDirDict(PointDict):
    __type_file__='<type:file>'
    __type_dir__='<type:dir>'
    __type_link__='<type:link>'
    def set_info(self,info):
        return self.seta(info=info)
    def get_info(self):
        return self.geta('info')
    def info(self,*args,**kwargs):
        if len(args)!=0:
            return self.get_info()[args[0]]
        if len(kwargs)!=0:
            info=self.get_info()
            info.update(**kwargs)
            return self.set_info(info)
        return self.get_info()
    def generate_info(self):
        path=self.path()
        info=path.info()
        info.update(path=path)
        info.update(abspath=path.abspath())
        self.set_info(info)
    def set_type(self,type):
        return self.set_attribute('__type__',type)
    def get_type(self,*args,**kwargs):
        return self.get_attribute('__type__',*args,**kwargs)
    def set_size(self,size):
        return self.set_attribute('__size__',size)
    def get_size(self,*args,**kwargs):
        return self.get_attribute('__size__',*args,**kwargs)
    def get_path(self):
        return self.geta('path')
    def set_path(self,path):
        return self.seta(path=path)
    def path(self):
        return self.get_path()
    def print_size(self):
        print(self.auto_size_str())
    def auto_size_str(self):
        size = self.get_size()
        return self.pretty_format_size(size)
    def pretty_format_size(self,size):
        def gen_str(size,type):
            if size%1==0:return '%d %s'%(size,type)
            return '%.2f %s'%(size,type)
        def inrange(s):
            if size>=0 and size <1000:
                return True
        if inrange(size):return gen_str(size,'Bytes')
        size/=1024
        if inrange(size):return gen_str(size,'KB')
        size/=1024
        if inrange(size):return gen_str(size,'MB')
        size/=1024
        if inrange(size):return gen_str(size,'GB')
        size/=1024
        if inrange(size):return gen_str(size,'TB')
        size/=1024
        return gen_str(size,'PB')
    def size_str(self,type='Bytes'):
        size=self.size_format(type=type)
        if type=='Bytes':
            return '%d %s'%(size,type)
        return '%.2f %s'%(size,type)
    def info_format(self):
        return self.format_info(self.info())
    def format_info(self,info):
        info2=PointDict()
        for fd in ['atime','ctime','mtime']:
            v=info.get(fd,self.__no_value__)
            if v:
                info2[fd]=self.format_time(v)
        fd='size'
        v=info.get(fd,self.__no_value__)
        if v:
            info2[fd] = self.pretty_format_size(v)
        info3=PointDict(info)
        info3.update(info2)
        return info3


    def size_format(self,type='Bytes'):
        size=self.get_size()
        # size=self.size()
        return self.format_size(size=size,type=type)
    def format_time(self,t):
        import time
        t=time.gmtime(t)
        return time.strftime('%Y-%m-%d %H:%M:%S',t)
    def format_size(self,size,type='Bytes'):
        if type=='Bytes':return size
        size/=1024
        if type=='KB':return size
        size/=1024
        if type=='MB':return size
        size/=1024
        if type=='GB':return size
        size/=1024
        if type=='TB':return size
        else:
            raise Exception('Type %s not supported.'%(type))
    def set_face(self,string):
        self.set_attribute('__face__',string)
    def default_face(self):
        items=[]
        for k,v in self.items():
            items.append('%s:%s'%(k,v))
        return '<%s>'%(','.join(items))
    def __repr__(self):
        s=self.get_attribute('__face__',None)
        if s:
            return s
        else:
            return self.default_face()
    def print2(self,depth=0,step=5,space_around_delimiter=0,fillchar=' ',cell_border='|',delimiter=':'):
        import re
        def len_zh(data):
            temp = re.findall('[^a-zA-Z0-9.]+', data)
            count = 0
            for i in temp:
                count += len(i)
            return (count)
        for k,v in self.items():
            for i in range(depth):
                print(fillchar*step,end='')
                print(cell_border,end='')
            print(k.rjust(step-len_zh(k),fillchar),end=' '*space_around_delimiter+delimiter+' '*space_around_delimiter)
            if not isinstance(v,PointDict):
                print(v)
            if v.path().isfile():
                print('\n',end='')
                v.info_format().print1(depth=depth+1,step=step,space_around_delimiter=space_around_delimiter,
                          cell_border=cell_border,fillchar=fillchar,delimiter=delimiter)
            else:
                print('\n',end='')
                v.print2(depth=depth+1,step=step,space_around_delimiter=space_around_delimiter,
                          cell_border=cell_border,fillchar=fillchar,delimiter=delimiter)
    def pprint2(self):
        return self.print2(step=5, space_around_delimiter=0, fillchar='`', cell_border='|', delimiter=':')

class DirTree(FileDirDict):
    def __init__(self,path):
        path=DirPath(path)
        self.set_path(path)
        self.generate_info()
        size=0
        n=0
        N=0
        for item in path():
            n+=1
            p2=path/item
            if p2.isdir():
                self[item]=DirTree(p2)
                N+=self[item].info('N')
            elif p2.isfile():
                N+=1
                file=FileDirDict(name=item)
                file.set_path(p2)
                file.generate_info()
                fsize = p2.getsize()
                file.info(size=fsize)
                file.set_size(size=fsize)
                file.update(size=file.auto_size_str())
                self[item]=file
            size+=self[item].info('size')
        self.info(size=size,n=n,N=N)
    def size(self):
        size=0
        for k,v in self.items():
            path=self.geta('path')/k
            if path.isfile():
                size+=path.size()
            elif path.isdir():
                size+=v.size()
        return size
    def pppprint(self):
        return self.pprint2()


class Status(PointDict):
    def __init__(self,success=True,message="success",code=0,data=None,*args,**kwargs):
        super().__init__(success=success,message=message,code=code,data=data,*args,**kwargs)
class StatusSuccess(Status):
    def __init__(self,success=True,message="success",code=0,data=None,*args,**kwargs):
        super().__init__(success=success,message=message,code=code,data=data,*args,**kwargs)
class StatusError(Status):
    def __init__(self,success=False,message="failure",code=-1,data=None,*args,**kwargs):
        super().__init__(success=success,message=message,code=code,data=data,*args,**kwargs)

class RuleChecker:
    def __init__(self,rules=[]):
        self.rules = []
        for rule in rules:
            self.define(*rule)
    def define(self,rule, res , do_in=None,do_out=None,match_func=None):
        self.rules.append([rule,res,do_in,do_out,match_func])
    def _match_rule(self,rule,x):
        if isinstance(rule,type):
            return isinstance(x,rule)
        elif hasattr(rule,'__call__'):
            return rule(x)
        else:
            return rule is x
    def __call__(self, x , default=None):
        for rule, res ,do_in,do_out,match_func in self.rules:
            match_func=match_func or self._match_rule
            if match_func(rule,do_in(x) if do_in else x):
                return do_out(res) if do_out else res
        return default

class DefaultDict(dict):
    default={

    }
    def __init__(self, **kwargs):
        def run_func_with_context(func, arg=None):
            # print(func)
            try:
                args = inspect.getfullargspec(func)
            except TypeError as e:
                return func()

            args = args.args
            if len(args):
                return func(arg)
            else:
                return func()

        info = kwargs
        for k, v in self.default.items():
            if k not in info.keys() or info[k] is None:
                info[k] = self.default[k] if not hasattr(self.default[k],
                                                              '__call__') else run_func_with_context(self.default[
                                                                                                         k], info)
        super().__init__(**info)

if __name__ == '__main__':
    ruler=RuleChecker([
        ['b',1],
        [lambda x:x>2 if isinstance(x,int) else False,2,]
    ])
    x='b'
    x=ruler(x)
    print(x)




from .fs_handles import FakeOS
import re

class FileSearchEngine(FakeOS):
    def __init__(self,path):
        super().__init__(path)
        self.files=self.get_all_files()
    def refresh_cache(self):
        self.files=self.get_all_files()
    def get_all_files(self):
        return self.glob('./**/*.*',recursive=True)
    def search(self,keywords,match_all=True):
        fs=self.files
        if match_all:
            for word in keywords:
                fs=list(filter(lambda f:re.findall(word,f),fs))
            return fs
        else:
            def match(text,ptns):
                for ptn in ptns:
                    if re.findall(ptn,text):return True
                return False
            fs=list(filter(lambda f:match(f,keywords),fs))
            return fs




import os
pkg_dir=os.path.dirname(__file__)
pkg_data_dir=pkg_dir+'/data'
pkg_scripts_dir=pkg_data_dir+'/shell_scripts'
pkg_documents_dir=pkg_data_dir+'/documents'
pkg_static_dir=pkg_data_dir+'/static'
pkg_templates_dir=pkg_data_dir+'/templates'

def is_linux():
    import sys
    pf=sys.platform
    if pf=='linux':
        return True
    return False

def is_windows():
    import sys
    pf=sys.platform
    if pf=='win32':
        return True
    return False




#!/usr/bin/python
# -*- coding:utf-8 -*-

import sys

from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtWebEngineWidgets import *

class Form(QWidget):
    def __init__(self, parent=None):
        super(Form, self).__init__(parent)

        # self.setWindowOpacity(1)
        # self.setWindowFlags(Qt.FramelessWindowHint)
        # self.setAttribute(Qt.WA_TranslucentBackground)
        # self.showFullScreen()
        rect = QApplication.desktop().screenGeometry()
        w=rect.width()
        h=rect.height()
        w=int(w*0.5)
        h=int(h*0.5)
        self.resize(w,h)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)

        self.webview = QWebEngineView()

        vbox = QVBoxLayout()
        vbox.addWidget(self.webview)

        main = QGridLayout()
        main.setSpacing(0)
        main.addLayout(vbox, 0, 0)

        self.setLayout(main)
        btn = QPushButton('关闭', self)
        # 关闭应用
        btn.clicked.connect(QCoreApplication.instance().quit)
        # self.setWindowTitle("CoDataHD")
        # webview.load(QUrl('http://www.cnblogs.com/misoag/archive/2013/01/09/2853515.html'))
        # webview.show()

    def load(self, url):
        # self.webview.load(QUrl(url))
        self.webview.setHtml(open(r'D:\work\wk\data/test.html').read())
        self.webview.show()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    screen = Form()
    screen.show()
    url = "https://www.baidu.com"
    screen.load(url)
    sys.exit(app.exec_())


def get_local_ip():
    import socket
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip



from flask import Flask, request, Blueprint, abort, send_file, redirect
from wpkit.web import resources, utils
from wpkit.pan import Pan, LocalFSHandle
from wpkit.web.base import MyBlueprint
from wpkit.web.resources import env, get_template_by_name
from wpkit.web.utils import parse_json_and_form,parse_cookies,parse_all
from wpkit.basic import Status,StatusError,StatusSuccess,PointDict
import logging, functools,json
import wpkit


class BluePan(MyBlueprint):
    add_to_sitemap = True
    def __init__(self, import_name=None, name='pan', datapath='./data/pan', url_prefix='/pan',
                  **kwargs):
        # github_path = "git@github.com:Peiiii/MyCloudSpace.git",
        super().__init__(name=name, import_name=import_name, url_prefix=url_prefix, **kwargs)
        self.datapath = wpkit.basic.DirPath(datapath)
        self.db = wpkit.piu.Piu(path=self.datapath.db)
        # print("db:",self.db.dic)
        self.panpath = self.datapath / 'pans'
        def user_panpath(email):
            return self.panpath/email
        # self.pan = None
        self.pans={}
        self.config_statics({
            "/files": self.panpath
        })
        self.usman = wpkit.web.utils.UserManager(dbpath=self.datapath.usman.db, home_url=self.url_prefix)
        usman = self.usman
        @self.usman.login_required
        @parse_all
        def getUrl(user_email,location, name):
            # print("getUrlK")
            res = 'http://%s:%s' % (
                self.app.host, self.app.port) + self.url_prefix + '/files/'+user_email+"/" + self.pans[user_email].local_path(
                location + '/' + name)
            print("res", res)
            return res

        rd_post_github_path=redirect(location=self.url_prefix+'/post_github_path')


        def check_pan(func):
            @parse_cookies
            @functools.wraps(func)
            def wrapper(user_email,*args, **kwargs):
                user=self.usman.get_user(user_email)

                if not user.get('initialized', None):
                    github_path=None
                    try:
                        github_path = user.get('github_path', None)
                        print("get github_path:",github_path)
                        if not github_path:
                            return rd_post_github_path
                        self.pans[user_email]= Pan.init(user_panpath(user_email), github_path=github_path)
                    except:
                        # raise
                        Pan.clear(user_panpath(user_email))
                        print("error occured when init pan, github_path:",github_path or None)
                        return redirect(location=self.url_prefix+'/post_github_path')
                    print('Initializing Pan at %s' % (self.panpath))
                    user.update(initialized=True)
                    self.usman.update_user(user_email,user)

                self.pans[user_email] = Pan(user_panpath(user_email))
                self.pans[user_email].add_cmd('getUrl', getUrl)
                return func(*args,**kwargs)
            return wrapper

        def check_github_path(func):
            @parse_cookies
            @functools.wraps(func)
            def wrapper(user_email,*args, **kwargs):
                user=usman.get_user(user_email)
                if not user.get('github_path',None):
                    return redirect(location=self.url_prefix+'/post_github_path')
                return func(*args, **kwargs)

            return wrapper

        self.route('/login', methods=['post'])(usman.login(redirect_to=redirect(location=self.url_prefix)))
        self.route('/login', methods=['get'])(usman.login_page)
        self.route('/signup', methods=['post'])(usman.signup)
        self.route('/signup', methods=['get'])(usman.signup_page)

        @self.route('/post_github_path',methods=['GET'])
        def do_get_post_github_path():
            # print("hello")
            # return abort(404)
            return get_template_by_name('form').render(
                action=self.url_prefix + '/post_github_path',
                method='POST',
                name='github_path',
                msg='Input github path'
            )
        @self.route('/post_github_path', methods=['POST'])
        @parse_all
        def do_post_github_path(user_email,github_path):
            user=self.usman.get_user(user_email)
            print("github_path:", github_path)
            if github_path:
                user.update(github_path=github_path)
                self.usman.update_user(user_email, user)
                return redirect(location=self.url_prefix)
            else:
                return "error"

        @self.route('/', methods=['get'])
        @usman.login_required
        @check_pan
        @check_github_path
        def do_pan_get():
            return resources.get_template_by_name('pan').render()

        @self.route('/', methods=['POST'])
        def do_pan_post():
            return

        @self.route('/cmd', methods=['post'])
        @usman.login_required
        @utils.parse_all
        def do_cmd(user_email,cmd):
            user=usman.get_user(user_email)
            print('cmd:', cmd)
            try:

                res = self.pans[user_email].execute(cmd)
                res=StatusSuccess(data=res)
            except:
                # raise
                res=StatusError()
            print('res:', res)
            return utils.jsonify(res)

        @self.route('/upload', methods=['POST', 'GET'])
        @self.usman.login_required
        @utils.parse_all
        def do_upload(user_email,info):
            file = request.files['file']
            if isinstance(info, str):
                info = json.loads(info)
            info = PointDict.from_dict(info)
            path = self.pans[user_email].local_path(info.location)
            path = self.pans[user_email].local_path(path + '/' + info.filename)
            path=user_panpath(user_email)+"/"+path
            file.save(path)
            print('path:', path)
            print('file:', file)
            return StatusSuccess(msg="Uploading succeeded.")



'''
prerequisites:
jinja2
'''


class Tem:
    def __init__(self, tem_file):
        from jinja2 import Template, Environment
        self.env = Environment()
        self.tem_file = tem_file
        self.tem = self.env.from_string(open(tem_file, 'r', encoding='utf-8').read())

    def render(self, *args, **kwargs):
        return self.tem.render(*args, **kwargs)

    def render_to_file(self, fp, *args, **kwargs):
        with open(fp, 'w', encoding='utf-8') as f:
            s = self.render(*args, **kwargs)
            f.write(s)
        return s


def gen_service(dst_file="./mysite", service_name='mysite', project_dir='/root/projects/mysite', script_file_name='main.py'):
    import pkg_resources
    tmp_path = pkg_resources.resource_filename('wk', 'data/templates/service.sh')
    tem = Tem(tmp_path)
    if dst_file:
        r = tem.render_to_file(fp=dst_file, service_name=service_name,
                               project_dir=project_dir,
                               script_file_name=script_file_name
                               )
    else:
        r = tem.render(service_name=service_name,
                       project_dir=project_dir,
                       script_file_name=script_file_name
                       )
    return r



import json
from flask import current_app
class JsonResponse(dict):
    def __init__(self, success=None,code=None,message=None,data=None,action=None,params=None):
        super().__init__(success=success,code=code,message=message,data=data,action=action,params=params)
    def jsonify(self):
        response = current_app.response_class(
            response=json.dumps(self,ensure_ascii=False,indent=2),
            status=200,
            mimetype='application/json'
        )
        return response
class ActionResponse(JsonResponse):
    def __init__(self,action,params={},data=None,message=None,success=None,code=None):
        super().__init__(action=action,params=params,data=data,message=message,success=success,code=code)

class ActionRedirect(ActionResponse):
    def __init__(self,location,success=True,message=None):
        params=dict(location=location)
        super().__init__(action='redirect',params=params,success=success,message=message)
class ActionRefresh(ActionResponse):
    def __init__(self,success=True,message=None):
        super().__init__(action='refresh',success=success,message=message)
class StatusResponse(JsonResponse):
    def __init__(self,success=True,message="success",code=0,data=None,*args,**kwargs):
        super().__init__(success=success,message=message,code=code,data=data,*args,**kwargs)
class StatusSuccessResponce(StatusResponse):
    def __init__(self,success=True,message="success",code=0,data=None,*args,**kwargs):
        super().__init__(success=success,message=message,code=code,data=data,*args,**kwargs)
class StatusErrorResponse(StatusResponse):
    def __init__(self,success=False,message="failure",code=-1,data=None,*args,**kwargs):
        super().__init__(success=success,message=message,code=code,data=data,*args,**kwargs)


import git as g
import os
from wk.basic import PowerDirPath,PointDict,join_path,standard_path

class LocalFSHandle:
    def __init__(self,path):
        os.makedirs(path) if not os.path.exists(path) else None
        assert os.path.exists(path)
        # print("path:",path)
        path=os.path.abspath(path)
        self.lpath=path
        self.curser=PowerDirPath(path)
        self.cmd_dict={
            'newFile':self.newFile,
            'newDir':self.newDir,
            'getFile':self.getFile,
            'getDir':self.getDir,
            'saveFile':self.saveFile,
            'delete':self.delete,
        }
    def add_cmd(self,cmd,func):
        self.cmd_dict[cmd]=func
    @classmethod
    def init(cls,path):
        os.makedirs(path) if not os.path.exists(path) else None
        return cls(path=path)
    @classmethod
    def clear(cls,path):
        PowerDirPath(path).rmself().todir()
    def true_path(self,path):
        return join_path(self.lpath,self.local_path(path))
    def local_path(self,path):
        # print(path)
        try:
            path=standard_path(path,check=True)
        except:
            return None
        return path
    def saveFile(self,filename,location,content):
        location = self.true_path(location)
        f = PowerDirPath(location)/filename
        return f(content)
    def newFile(self,filename,location,content=None):
        location = self.true_path(location)
        loc=PowerDirPath(location)
        return loc.file(filename)(content) if content is not None else loc.file(filename)
    def newDir(self,dirname,location):
        location = self.true_path(location)
        loc = PowerDirPath(location)
        return loc(dirname)
    def delete(self,name,location):
        location = self.true_path(location)
        loc = PowerDirPath(location)
        return (loc/name).rmself()
    def getFile(self,filename,location):
        location = self.true_path(location)
        loc = PowerDirPath(location)
        return (loc/filename)()
    def getDir(self,dirname,location):
        location=self.true_path(location)
        loc = PowerDirPath(location)
        li=(loc/dirname)()
        return [{'name':i,'type':PowerDirPath(loc/dirname/i).type()} for i in li]
    def execute(self,cmd):
        cmd=PointDict.from_dict(cmd)
        op,params=cmd.op,cmd.params
        # if 'location' in params.keys():
        #     params['location']=self.true_path(params['location'])
        if op in self.cmd_dict.keys():
            # print(self.cmd_dict[op](**params))
            # print("params:",params)
            res= self.cmd_dict[op](**params)
            # print("res:",res)
            return res

class Pan(LocalFSHandle):
    def __init__(self,path):
        super().__init__(path)
        self.repo = g.Repo(path)
        git = self.repo.git
        self.git = git
        self.add_cmd('pull',self.pull)
        self.add_cmd('push',self.push)
        self.add_cmd('goback',self.goback)
    @classmethod
    def init(cls, path, github_path):
        os.makedirs(path) if not os.path.exists(path) else None
        repo = g.Repo.init(path)
        git = repo.git
        try:
            git.remote('add', 'origin', github_path)
        except:
            pass
        git.pull('origin', 'master')
        git.config("--global","push.default","simple")
        # git.push("--set-upstream","origin","master")
        # git.branch('--set-upstream-to=origin/master', 'master')
        return cls(path=path)
    def pull(self):
        git=self.git
        git.fetch('--all')
        git.reset('--hard','origin/master')
    def push(self):
        git=self.git
        git.add('.')
        git.commit('-m','test')
        # git.push('origin','master')
        git.push("--set-upstream", "origin", "master")
        # git.push("--set-upstream", "origin", "master",">","log.txt")
    def goback(self,n=1):
        self.git.reset('--hard','HEAD'+'^'*n)
    def destroy(self):
        try:
            PowerDirPath(self.lpath).rmself()
        except:
            pass





demo_code=\
'''
pan = Pan.init('./myspace', github_path='http://github.com/Peiiii/MyCloudSpace')
pan=Pan('./myspace')
repo.pull()
repo.goback(4)
a=pan.getDir('./',location='./myspace')
'''



import time,functools

class Timer:
    def __init__(self, verbose=True,msg=None,mute=None):
        self.history = []
        self.dt_history = []
        self.steps = 0
        self.start_time = time.time()
        self.history.append(self.start_time)
        self.verbose = verbose
        self.mute=mute
        if self.verbose:
            self.print('Timer started at %s' % (self.start_time))
        if msg:
            self.print(msg)
    def print(self,*args,**kwargs):
        if not self.mute:
            print(*args,**kwargs)
    def step(self,msg=None):
        t = time.time()
        dt = t - self.history[-1]
        self.dt_history.append(dt)
        self.history.append(t)
        self.steps += 1
        if self.verbose:
            self.print('step=%s , %s time since last step: %s' % (self.steps,'msg=%s'%(msg) if msg else '',dt))
        return dt
    def mean(self):
        if not len(self.dt_history):return None
        return sum(self.dt_history)/len(self.dt_history)
    def plot_dt_history(self,title='Timer History',*args,**kwargs):
        from matplotlib import pyplot as plt
        plt.plot(self.dt_history)
        plt.show(title=title,*args,**kwargs)

    def end(self):
        t = time.time()
        self.end_time = t
        dt = t - self.history[-1]
        self.dt_history.append(dt)
        self.history.append(t)
        self.steps += 1
        if self.verbose:
            self.print('time since last step: %s' % (dt))
        return dt
DEFALUT_TIMER=None
def timer_step(msg=None):
    global DEFALUT_TIMER
    if not DEFALUT_TIMER:
        DEFALUT_TIMER=Timer()
    else:
        DEFALUT_TIMER.step(msg)


def run_timer(func):
    name = func.__name__

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        print('running %s ...' % (name))
        t = Timer(verbose=False)
        ret = func(*args, **kwargs)
        dt = t.end()
        print('finished running %s ,time consumed: %s' % (name, dt))
        return ret
    return wrapper

